import os
from typing import Optional

import psutil
from eva.pltform.evaluation.core import EvaluatorOptionsDefaults
from eva.pltform.rt.dockert import DockerRuntimeDefaults
from pydantic import BaseModel, model_validator


def mem_limit_to_bytes(mem_limit_str):
    mem_limit_str = mem_limit_str.strip().lower()
    if mem_limit_str.endswith('m'):
        return int(mem_limit_str[:-1]) * 1024 * 1024
    elif mem_limit_str.endswith('g'):
        return int(mem_limit_str[:-1]) * 1024 * 1024 * 1024
    elif mem_limit_str.endswith('k'):
        return int(mem_limit_str[:-1]) * 1024
    else:
        return int(mem_limit_str)


def getenv(var, default=None, cast=None):
    val = os.getenv(var, default)
    return cast(val) if cast is not None and val is not None else val


class Config(BaseModel):
    log_level: str
    rabbitmq_host: str
    registry_url: str

    docker_max_instances: int
    docker_mem_limit: str
    docker_cpu_quota: float
    docker_network_mode: str
    docker_img_cache_ttl: int
    docker_img_base_path: str
    docker_img_registry: Optional[str]
    docker_img_registry_user: Optional[str]
    docker_img_registry_password: Optional[str]

    compile_timeout: Optional[int]
    test_timeout: Optional[int]
    style_timeout: Optional[int]
    codecov_timeout: Optional[int]
    max_output_size: Optional[int]

    @staticmethod
    def from_env():

        return Config(
            rabbitmq_host=getenv('EVA_DISPATCHER_RABBITMQ_HOST', 'localhost'),
            log_level=getenv('EVA_DISPATCHER_LOG_LEVEL', 'INFO'),
            registry_url=getenv('EVA_DISPATCHER_REGISTRY_URL', 'http://localhost:7777'),
            docker_max_instances=getenv(
                'EVA_DISPATCHER_DOCKER_MAX_INSTANCES', DockerRuntimeDefaults.MAX_INSTANCES, cast=int
            ),
            docker_mem_limit=getenv(
                'EVA_DISPATCHER_DOCKER_MEM_LIMIT', DockerRuntimeDefaults.MEM_LIMIT
            ),
            docker_cpu_quota=getenv(
                'EVA_DISPATCHER_DOCKER_CPU_QUOTA', DockerRuntimeDefaults.CPU_QUOTA, cast=float
            ),
            docker_network_mode=os.getenv(
                'EVA_DISPATCHER_DOCKER_NETWORK_MODE', DockerRuntimeDefaults.NETWORK_MODE
            ),
            docker_img_cache_ttl=getenv(
                'EVA_DISPATCHER_DOCKER_IMG_CACHE_TTL', DockerRuntimeDefaults.IMG_CACHE_TTL, cast=int
            ),
            docker_img_base_path=getenv(
                'EVA_DISPATCHER_DOCKER_IMG_BASE_PATH', DockerRuntimeDefaults.IMG_BASE_PATH
            ),
            docker_img_registry=getenv(
                'EVA_DISPATCHER_DOCKER_IMG_REGISTRY', DockerRuntimeDefaults.IMG_REGISTRY
            ),
            docker_img_registry_user=getenv(
                'EVA_DISPATCHER_DOCKER_IMG_REGISTRY_USER', DockerRuntimeDefaults.IMG_REGISTRY_USER
            ),
            docker_img_registry_password=getenv(
                'EVA_DISPATCHER_DOCKER_IMG_REGISTRY_PASSWORD',
                DockerRuntimeDefaults.IMG_REGISTRY_PASSWORD,
            ),
            compile_timeout=getenv(
                'EVA_DISPATCHER_EV_COMPILE_TIMEOUT',
                EvaluatorOptionsDefaults.COMPILE_TIMEOUT,
                cast=int,
            ),
            test_timeout=getenv(
                'EVA_DISPATCHER_EV_TEST_TIMEOUT', EvaluatorOptionsDefaults.TEST_TIMEOUT, cast=int
            ),
            style_timeout=getenv(
                'EVA_DISPATCHER_EV_STYLE_TIMEOUT', EvaluatorOptionsDefaults.STYLE_TIMEOUT, cast=int
            ),
            codecov_timeout=getenv(
                'EVA_DISPATCHER_EV_CODECOV_TIMEOUT',
                EvaluatorOptionsDefaults.CODECOV_TIMEOUT,
                cast=int,
            ),
            max_output_size=getenv(
                'EVA_DISPATCHER_EV_MAX_OUTPUT_SIZE',
                EvaluatorOptionsDefaults.MAX_OUTPUT_SIZE,
                cast=int,
            ),
        )

    @model_validator(mode='after')
    def check_cpu_quota(self):
        avail_cpus = psutil.cpu_count(logical=True)
        total_cpu_quota = self.docker_cpu_quota * self.docker_max_instances
        if total_cpu_quota > avail_cpus:
            raise ValueError(
                f'configured total cpu quota is not valid. '
                f'{self.docker_cpu_quota} (cpu quota per evaluator) * '
                f'{self.docker_max_instances} (max instances) = '
                f'{total_cpu_quota} > {avail_cpus} (available physical cpus)'
            )

        return self

    @model_validator(mode='after')
    def check_mem_quota(self):
        avail_mem = psutil.virtual_memory()[0]
        total_mem = mem_limit_to_bytes(self.docker_mem_limit) * self.docker_max_instances
        if total_mem > avail_mem:
            raise ValueError(
                f'configured total memory quota is not valid. '
                f'{self.docker_mem_limit} (memory quota per evaluator) * '
                f'{self.docker_max_instances} (max instances) = '
                f'{total_mem} bytes > {avail_mem} (available memory)'
            )

        return self
