import loguru
from eva.dispatcher.dto import EvalRequest, RequestedCapability
from eva.dispatcher.registry import EvaluationStack, EvaluatorDeclaration, RegistryClient
from eva.pltform import EvaluatorRegistry
from eva.pltform.evaluation.core import EvalPayload, EvaluationContext, EvaluatorOptions
from eva.pltform.rt import Runtime


class Dispatcher:
    def __init__(
        self,
        runtime: Runtime,
        registry: RegistryClient,
        evaluator_opts: EvaluatorOptions = EvaluatorOptions(),
    ):
        self.runtime = runtime
        self.registry = registry
        self.evaluator_opts = evaluator_opts

    def get_default_capability(self, stack: EvaluationStack):
        return RequestedCapability(
            compile=stack.evaluator.capability.compile,
            style=stack.evaluator.capability.style,
            test=stack.evaluator.capability.test,
            coverage=False,
        )

    def create_payload(self, request: EvalRequest, stack: EvaluationStack):
        capability = (
            request.capability
            if request.capability is not None
            else self.get_default_capability(stack)
        )

        return EvalPayload(
            solution_files=request.solution_files,
            test_files=request.test_files,
            include_files=request.include_files,
            compile=capability.compile,
            style=capability.style,
            test=capability.test,
            params=request.params,
        )

    def validate_request(self, request: EvalRequest, stack: EvaluationStack):
        if request.capability is None:
            return

        evaluator = stack.evaluator

        if request.capability.compile and not evaluator.capability.compile:
            raise RuntimeError(f'evaluator {evaluator} does not support compilation')

        if request.capability.style and not evaluator.capability.style:
            raise RuntimeError(f'evaluator {evaluator} does not support style checks')

        if request.capability.test and not evaluator.capability.test:
            raise RuntimeError(f'evaluator {evaluator} does not support tests')

        if (
            evaluator.capability.compile
            and request.capability.test
            and not request.capability.compile
        ):
            raise RuntimeError('compilation is required to run tests')

    def create_evaluator(self, evaluator: EvaluatorDeclaration):
        registry = EvaluatorRegistry()
        clazz = registry.find_by_id(evaluator.id)
        if clazz is None:
            raise RuntimeError(f'unknown evaluator: {evaluator.id}')

        return clazz()

    def process(self, request: EvalRequest, request_logger: 'loguru.Logger' = None):
        request_logger = request_logger if request_logger is not None else loguru.logger

        request_logger.debug(f'Request: {request.model_dump_json(indent=4)}')

        response = self.registry.get_stack(request.stack_ref)
        request_logger.debug(
            f'Resolved stack: {response.stack}, last registry update: {response.last_update}'
        )

        stack = response.stack
        self.validate_request(request, stack)
        payload = self.create_payload(request, stack)

        request_logger.debug(
            f'Final configuration for the docker runtime: {payload.model_dump_json(indent=4)}'
        )

        evaluator = self.create_evaluator(stack.evaluator)
        with self.runtime.create(stack.environment.id, request_logger) as env:
            request_logger.debug('environment prepared for evaluation')

            ctx = EvaluationContext(
                payload=payload, env=env, opts=self.evaluator_opts, logger=request_logger
            )
            result = evaluator.eval(ctx)

            compiling_status = result.compilation.compiling if result.compilation else 'n/a'
            test_status = f'{result.test.passed}/{result.test.testcases}' if result.test else 'n/a'
            style_errors = result.style.errors if result.style else 'n/a'
            coverage_tool = (
                result.test.coverage.tool if result.test and result.test.coverage else 'n/a'
            )

            request_logger.info(
                f'[{stack.ref}] '
                f'compiling={compiling_status}, '
                f'tests={test_status}, '
                f'style_errors={style_errors}, '
                f'coverage={coverage_tool}'
            )

            return result

    def shutdown(self):
        self.runtime.shutdown()
