import asyncio
import json
import signal
from concurrent.futures import ThreadPoolExecutor

import semantic_version
from aio_pika import Message, connect_robust
from aio_pika.abc import AbstractIncomingMessage
from eva.dispatcher.config import Config
from eva.dispatcher.const import VERSION
from eva.dispatcher.dispatcher import Dispatcher
from eva.dispatcher.dto import EvalRequest, EvalResponse, SysConfig
from eva.dispatcher.logsetup import setup_logging
from eva.dispatcher.registry import RegistryClient
from eva.pltform.evaluation.core import EvaluatorOptions
from eva.pltform.rt import RuntimeInterruptedError
from eva.pltform.rt.dockert import DockerRuntime
from loguru import logger


class RabbitMQHandler:
    # in bytes. Don't forget to also update the size limit in rabbitmq config
    MAX_MESSAGE_SIZE = 128 * 1024 * 1024

    HEADER_VERSION = 'VERSION'

    def __init__(self, host, dispatcher, prefetch):
        self.host = host
        self.channel = None
        self.dispatcher = dispatcher
        self.future: asyncio.Future = None
        self.tasks = []
        self.executor = ThreadPoolExecutor()
        self.prefetch = prefetch

    def shutdown(self):
        logger.info(
            'Shutting down. Please wait before pressing CTRL + C again to let the '
            'system clean up pending docker containers!'
        )
        self.future.cancel()

    def parse_request(self, message: AbstractIncomingMessage) -> EvalRequest:
        raw_request = message.body.decode()
        logger.debug(f'Parsing request: {raw_request}')

        request = json.loads(raw_request)
        return EvalRequest(**request)

    async def send_response(self, message: AbstractIncomingMessage, response: EvalResponse):
        raw_response = response.model_dump_json().encode()

        # check for responses that exceed the configured size limit in rabbitmq
        if len(raw_response) > self.MAX_MESSAGE_SIZE:
            raise RuntimeError(f'response too large: {len(raw_response)} > {self.MAX_MESSAGE_SIZE}')

        if message.reply_to is not None:
            default_exchange = self.channel.default_exchange
            await default_exchange.publish(
                Message(
                    body=raw_response,
                    correlation_id=message.correlation_id,
                ),
                routing_key=message.reply_to,
            )

    def check_version(self, message: AbstractIncomingMessage):

        if self.HEADER_VERSION not in message.properties.headers:
            raise RuntimeError(
                f'{self.HEADER_VERSION} missing in header of the message but is required'
            )

        v = message.properties.headers.get(self.HEADER_VERSION)
        try:
            specified = semantic_version.Version(v)
        except ValueError as e:
            raise RuntimeError(
                f'Specified version {v} in {self.HEADER_VERSION} header is invalid'
            ) from e

        current = semantic_version.Version(VERSION)
        if specified.major != current.major:
            raise RuntimeError(
                f'Specified version {v} in {self.HEADER_VERSION} header is not compatible '
                f'with the current version {VERSION}'
            )

    async def run(self):
        self.future = asyncio.Future()

        # connect_robust does not reconnect if rabbitmq is not available on startup.
        # however, it will try to connect again, if the connection is lost after it has been
        # established once
        while True:
            try:
                connection = await connect_robust(f'amqp://{self.host}/')
                break
            except ConnectionError:
                logger.error(
                    f'No connection to RabbitMQ broker (host={self.host}), '
                    f'trying again in 5 seconds.'
                )
                await asyncio.sleep(5)

        logger.info('Connected to RabbitMQ broker.')
        self.channel = await connection.channel()
        await self.channel.set_qos(prefetch_count=self.prefetch)
        exchange = await self.channel.get_exchange('amq.topic')

        queue = await self.channel.declare_queue(
            'processEvaRequests', arguments={'x-expires': 15 * 60 * 1000}
        )
        await queue.bind(exchange, routing_key='eva.request')
        logger.info('Queue prepared.')

        async def process_message(
            message: AbstractIncomingMessage,
        ) -> None:

            # don't accept new messages if shutdown started
            if self.future.cancelled():
                return

            sysconf = SysConfig(
                rt=self.dispatcher.runtime.describe(), evaluator_opts=self.dispatcher.evaluator_opts
            )

            async with message.process(ignore_processed=True):
                request_id = message.correlation_id if message.correlation_id else 'n/a'
                request_logger = logger.bind(request_id=request_id)

                try:
                    request_logger.info('Processing request...')

                    self.check_version(message)
                    request = self.parse_request(message)

                    # we use a thread pool because tasks are i/o bound and
                    # asyncio cannot be used in the dispatcher implementation
                    # (because we use streamlit in the demo)
                    loop = asyncio.get_event_loop()

                    task = loop.run_in_executor(
                        self.executor, self.dispatcher.process, request, request_logger
                    )
                    self.tasks.append(task)
                    result = await task
                    self.tasks.remove(task)

                    response = EvalResponse(
                        success=True, result=result, version=VERSION, sysconf=sysconf
                    )
                    await self.send_response(message, response)

                    await message.ack()
                except RuntimeInterruptedError:
                    await message.reject(True)  # requeue
                # pylint: disable=broad-exception-caught
                except Exception as e:
                    await message.ack()
                    request_logger.exception('Failed to process message')
                    response = EvalResponse(
                        success=False, message=str(e), version=VERSION, sysconf=sysconf
                    )
                    await self.send_response(message, response)

        await queue.consume(process_message)

        try:
            await self.future
        finally:
            self.dispatcher.shutdown()

            # we use gather to surpress these exceptions:
            # https://docs.python.org/3/library/asyncio-dev.html#detect-never-retrieved-exceptions
            await asyncio.gather(*self.tasks, return_exceptions=True)
            self.executor.shutdown(wait=True, cancel_futures=True)

            # _sometimes_ there are still unretrieved exceptions, race condition?
            # lets see if it helps to do it again:
            await asyncio.gather(*self.tasks, return_exceptions=True)


def main():
    config = Config.from_env()
    setup_logging(level=config.log_level, with_files=True)
    logger.info(
        f'config: {config.model_dump_json(indent=4, exclude=['docker_img_registry_password'])}'
    )

    evaluator_opts = EvaluatorOptions(
        compile_timeout=config.compile_timeout,
        style_timeout=config.style_timeout,
        test_timeout=config.test_timeout,
        codecov_timeout=config.codecov_timeout,
        max_output_size=config.max_output_size,
    )
    docker = DockerRuntime(
        max_instances=config.docker_max_instances,
        cpu_quota=config.docker_cpu_quota,
        mem_limit=config.docker_mem_limit,
        network_mode=config.docker_network_mode,
        img_cache_ttl=config.docker_img_cache_ttl,
        img_base_path=config.docker_img_base_path,
        img_registry=config.docker_img_registry,
        img_registry_user=config.docker_img_registry_user,
        img_registry_password=config.docker_img_registry_password,
    )
    registry = RegistryClient(config.registry_url)
    dispatcher = Dispatcher(runtime=docker, registry=registry, evaluator_opts=evaluator_opts)
    handler = RabbitMQHandler(
        host=config.rabbitmq_host, dispatcher=dispatcher, prefetch=config.docker_max_instances + 2
    )

    signal.signal(signal.SIGINT, lambda *args: handler.shutdown())
    signal.signal(signal.SIGTERM, lambda *args: handler.shutdown())

    try:
        asyncio.run(handler.run())
    except asyncio.exceptions.CancelledError:
        logger.info('Shutdown complete.')


if __name__ == '__main__':
    main()
