import os

import streamlit as st
from eva.playground.components import files_tabset, render_result, request_form
from eva.playground.const import VERSION
from eva.playground.dispatcher.client import EvaluationFailedError
from eva.playground.dispatcher.dto import EvalRequest, RequestedCapability
from eva.playground.resources import get_dispatcher, get_registry, get_stack, get_stacks, get_tasks
from eva.playground.state import state
from eva.playground.util import capture_logs
from loguru import logger

st.set_page_config(
    page_title='Eva Playground',
    page_icon='🧪',
    layout='wide',
)

st.title('Eva Playground')
st.caption(f'Version: {VERSION}')

show_ar_warning = os.environ.get('EVA_PG_SHOW_AR_WARNING', 'true').lower() == 'true'
if show_ar_warning:
    st.info(
        'Note that auto-reloading (hot reloading) '
        'may lead to problems with session state serialization. See the docs for more info.'
    )

tasks_map = get_tasks(os.getenv('EVA_PG_TASK_POOLS_DIR', os.path.dirname(__file__)))
pool = st.selectbox(
    'Pool', tasks_map.keys(), format_func=lambda t: t.id, on_change=state.clear_files
)

task = st.selectbox(
    'Task',
    pool.tasks,
    format_func=lambda t: f'{t.meta.name} ({t.meta.evaluator})',
    on_change=state.clear_files,
)

files = state.files(task)
files_tabset(files)

registry = get_registry()
stacks = get_stacks(registry)

matching_stack = get_stack(registry, task.meta.evaluator)
idx = next(iter([idx for idx, s in enumerate(stacks) if s.ref == matching_stack.ref]), 0)

stack = st.selectbox('Stack', stacks, format_func=lambda t: t.ref, index=idx)

evaluation = state.evaluation()

run, form = request_form(task, stack)
if run:
    with st.spinner('Evaluating...'):

        with capture_logs(form.log_level) as logs:
            capability = RequestedCapability(compile=form.compile, style=form.style, test=form.test)
            params = form.params
            if form.coverage:
                params = {**params, 'coverage.enabled': 'true', 'coverage.tool': form.coverage_tool}

            st.text('Final parameters:')
            st.write(params)

            req = EvalRequest(
                stack_ref=stack.ref,
                solution_files=[files.solution_file],
                include_files=files.include_files,
                test_files=files.test_files,
                capability=capability,
                params=params,
            )

            evaluation.req = req

            try:
                result = get_dispatcher().call(req)
                evaluation.result = result
                evaluation.error = None
            except EvaluationFailedError as e:
                # pylint: disable=invalid-name
                error = str(e)
                evaluation.result = None
                evaluation.error = error
                logger.error(error)

            evaluation.logs = logs.getvalue()

st.divider()
render_result(evaluation)
