import json
import threading

import amqpstorm
from amqpstorm import Message
from eva.playground.const import VERSION
from eva.playground.dispatcher.dto import EvalRequest, EvalResponse, EvalResult
from loguru import logger


class EvaluationFailedError(RuntimeError):
    pass


class DispatcherClient:
    def __init__(self, rabbitmq_host, timeout=15):
        self.rabbitmq_host = rabbitmq_host
        self.channel = None
        self.response = None
        self.connection = None
        self.callback_queue = None
        self.timeout = timeout
        self.results = {}
        self.events = {}
        self.open()

    def open(self):
        self.connection = amqpstorm.Connection(
            self.rabbitmq_host, username='guest', password='guest'
        )
        self.channel = self.connection.channel()

        result = self.channel.queue.declare(exclusive=True)
        self.callback_queue = result['queue']

        self.channel.basic.consume(self._on_response, no_ack=True, queue=self.callback_queue)

        self._create_process_thread()

    def _create_process_thread(self):
        """Create a thread responsible for consuming messages in response
        RPC requests.
        """
        thread = threading.Thread(target=self._process_data_events)
        thread.daemon = True
        thread.start()

    def _process_data_events(self):
        """Process Data Events using the Process Thread."""
        # self.channel.process_data_events()
        self.channel.start_consuming()

    def close(self):
        self.channel.stop_consuming()
        self.channel.close()
        self.connection.close()

    def call(self, req: EvalRequest) -> EvalResult:
        logger.info(f'Sending request: {req.model_dump_json(indent=4)}')

        headers = {'VERSION': VERSION}

        message = Message.create(
            self.channel, body=req.model_dump_json(), properties={'headers': headers}
        )
        message.reply_to = self.callback_queue
        correlation_id = message.correlation_id
        logger.debug(f'correlation_id: {correlation_id}')

        self.results[correlation_id] = None
        message.publish(exchange='amq.topic', routing_key='eva.request')

        event = threading.Event()
        self.events[message.correlation_id] = event
        if not event.wait(timeout=self.timeout):
            raise EvaluationFailedError('Timeout reached while requesting dispatcher')

        response = EvalResponse(**json.loads(self.results[correlation_id]))
        logger.debug(f'Received response: {response.model_dump_json(indent=4)}')

        if not response.success:
            raise EvaluationFailedError(f'Dispatcher returned an error: {response.message}')

        return response.result

    def _on_response(self, message):
        if message.correlation_id is None:
            logger.warning(f'Received message without correlation_id header: {message.body}')
            return

        logger.debug(f'received message with correlation_id: {message.correlation_id}')
        self.results[message.correlation_id] = message.body
        self.events[message.correlation_id].set()
