import os
import typing
from dataclasses import dataclass
from typing import Optional

import loguru
from eva.pltform.evaluation.core.payload import EvalPayload
from eva.pltform.evaluation.core.result import (
    CompilationResult,
    EvalResult,
    StyleResult,
    TestcaseStats,
    TestResult,
)
from eva.pltform.format.cexf import Coverage
from eva.pltform.format.trex import Testsuite
from eva.pltform.rt import EnvironmentHandle
from pydantic import BaseModel


class EvaluatorOptionsDefaults:
    COMPILE_TIMEOUT = 20
    STYLE_TIMEOUT = 10
    TEST_TIMEOUT = 20
    CODECOV_TIMEOUT = 30
    MAX_OUTPUT_SIZE = 4096


class EvaluatorOptions(BaseModel):
    max_output_size: int = EvaluatorOptionsDefaults.MAX_OUTPUT_SIZE
    compile_timeout: int = EvaluatorOptionsDefaults.COMPILE_TIMEOUT
    style_timeout: int = EvaluatorOptionsDefaults.STYLE_TIMEOUT
    test_timeout: int = EvaluatorOptionsDefaults.TEST_TIMEOUT
    codecov_timeout: int = EvaluatorOptionsDefaults.CODECOV_TIMEOUT


class StyleHandler:
    def run(self, evaluator: 'Evaluator', ctx: 'EvaluationContext') -> StyleResult:
        raise NotImplementedError()


class CompileHandler:
    def run(self, evaluator: 'Evaluator', ctx: 'EvaluationContext') -> CompilationResult:
        raise NotImplementedError()


class TestResultMapper:
    def map(self, testsuite: Testsuite, output: str, coverage: Coverage = None) -> TestResult:
        return TestResult(
            output=output,
            testcases=len(testsuite.testcases),
            passed=testsuite.passed(),
            failed=testsuite.failed(),
            stats=self._stats(testsuite),
            correctness=testsuite.correctness(),
            leaks=testsuite.leaks(),
            testsuite=testsuite,
            coverage=coverage,
        )

    def _stats(self, testsuite: Testsuite) -> TestcaseStats:
        return TestcaseStats(
            pending=testsuite.pending(),
            timeouts=testsuite.timeouts(),
            failures=testsuite.failures(),
            errors=testsuite.errors(),
        )


class TestHandler:
    mapper = TestResultMapper()

    def run(self, evaluator: 'Evaluator', ctx: 'EvaluationContext') -> TestResult:
        """
        Run the tests for the solution in the given context and return the test result.

        Notes:

        This method should **throw an exception only if there is a failure in the evaluator**.
        If solutions fail to meet assertions or do not handle exceptions,
        then this is not an evaluator failure. In this case, a valid test result should be returned.
        However, C and C++ programs may exit with segmentation fault, which may kill the test
        runner.
        As such, distinguishing between evaluator failures and solution failures is hard and may
        not always be possible.
        Safe wrappers (like the java wrapper in the cwithjava evaluator) may help to solve this
        problem.

        If the solution prints to stdout in infinite loops, the container may crash (out of
        memory) or the dispatcher
        may fail to send back the response because the response can grow to be dozens of
        megabytes large.
        Thus, **outputs need to be truncated if they are too long**.

        This method has to **return a valid test result if the solution does not terminate**.
        This allows to see the testcases that would have been executed.
        Additionally, this allows to identify the cause, i.e. infinite loops.

        :param evaluator:
        :param ctx:
        :return:
        """
        raise NotImplementedError()


@dataclass
class ExecutionHandlerConfig:
    style: Optional[StyleHandler] = None
    compile: Optional[CompileHandler] = None
    test: Optional[TestHandler] = None


@dataclass
class EvaluationContext:
    payload: EvalPayload
    env: EnvironmentHandle
    opts: EvaluatorOptions
    logger: 'loguru.Logger'


class EvaluatorCapability(BaseModel):
    compile: bool = False
    style: bool = False
    test: bool = False


class Evaluator:
    id: str
    supported_envs: Optional[str] = None
    excluded_envs: Optional[str] = None
    default_env: Optional[str] = None
    capability: EvaluatorCapability

    def setup(self, ctx: EvaluationContext):
        # pylint: disable=unused-argument
        return ExecutionHandlerConfig(
            compile=CompileHandler(), test=TestHandler(), style=StyleHandler()
        )

    def eval(self, ctx: EvaluationContext) -> EvalResult:
        files = ctx.payload.solution_files + ctx.payload.include_files + ctx.payload.test_files
        for f in files:
            path = os.path.join(ctx.env.cwd, f.path)
            ctx.env.create_file(f.content, path)

        handlers = self.setup(ctx)

        style_result: typing.Optional[StyleResult] = None
        if ctx.payload.style:
            style_result = handlers.style.run(self, ctx)
            ctx.logger.debug(
                f'[{handlers.compile.__class__.__name__}] '
                f'{style_result.model_dump_json(indent=4) if style_result is not None else None}'
            )

        compilation_result: typing.Optional[CompilationResult] = None
        if ctx.payload.compile:
            compilation_result = handlers.compile.run(self, ctx)
            ctx.logger.debug(
                f'[{handlers.compile.__class__.__name__}] '
                f'{compilation_result.model_dump_json(indent=4)}'
            )

        test_result: typing.Optional[TestResult] = None
        if ctx.payload.test and (not ctx.payload.compile or compilation_result.compiling):
            test_result = handlers.test.run(self, ctx)
            ctx.logger.debug(
                f'[{handlers.test.__class__.__name__}] {test_result.model_dump_json(indent=4)}'
            )

        result = EvalResult(compilation=compilation_result, test=test_result, style=style_result)
        return result
