import re
from typing import List, Optional

from eva.pltform.evaluation.core import EvaluationContext
from eva.pltform.evaluation.java.params import (
    JAVA_PARAM_ENABLE_ASSERTIONS,
    JAVA_PARAM_JAVAFX_MODULES,
)


class JavaArgBuilder:
    assertions: bool = False
    preview: bool = False
    max_ram: Optional[int] = None
    module_path: Optional[str] = None
    modules: List[str] = []

    def with_assertions(self, val):
        self.assertions = val
        return self

    def with_preview(self, val):
        self.preview = val
        return self

    def with_max_ram(self, val):
        self.max_ram = val
        return self

    def with_module_path(self, val):
        self.module_path = val
        return self

    def with_modules(self, val):
        self.modules = val
        return self

    def build(self):
        args = []
        if self.preview:
            args += ['--enable-preview']

        if self.assertions:
            args += ['-ea']

        if self.max_ram is not None:
            args += [f'-XX:MaxRAMPercentage={self.max_ram}']

        if self.module_path:
            args += [f'--module-path {self.module_path}']

        if len(self.modules) != 0:
            args += [f"--add-modules {','.join(self.modules)}"]

        return ' '.join(args)


class JavacArgBuilder:
    preview: bool = False
    release_version: Optional[int] = None
    lint: bool = False
    encoding: Optional[str] = None
    module_path: Optional[str] = None
    modules: List[str] = []

    def with_preview(self, val, release_version):
        self.preview = val
        self.release_version = release_version
        return self

    def with_lint(self, val):
        self.lint = val
        return self

    def with_encoding(self, val):
        self.encoding = val
        return self

    def with_module_path(self, val):
        self.module_path = val
        return self

    def with_modules(self, val):
        self.modules = val
        return self

    def build(self):
        args = []
        if self.preview:
            args += ['--enable-preview', f'--release {self.release_version}']

        if self.lint:
            args += ['-Xlint:preview -Xlint:deprecation -Xlint:unchecked']

        if self.encoding:
            args += [f'-encoding {self.encoding}']

        if self.module_path:
            args += [f'--module-path {self.module_path}']

        if len(self.modules) != 0:
            args += [f"--add-modules {','.join(self.modules)}"]

        return ' '.join(args)


def configure_jvm(ctx: EvaluationContext):
    java_args = JavaArgBuilder().with_max_ram(75).with_preview(True)

    if JAVA_PARAM_ENABLE_ASSERTIONS in ctx.payload.params:
        enable_assertions = ctx.payload.params.get(JAVA_PARAM_ENABLE_ASSERTIONS).lower() == 'true'
        java_args.with_assertions(enable_assertions)
    else:
        java_args.with_assertions(False)

    # todo: declare a LABEL 'JAVA_VERSION' in the docker image?
    match = re.search(r'java(\d+)', ctx.env.environment_id)
    if not match:
        raise RuntimeError(
            'configuring the jvm failed: does the id of the environment conform to the name '
            'java<ver>?'
        )
    java_version = int(match[1])

    javac_args = (
        JavacArgBuilder().with_lint(True).with_encoding('utf8').with_preview(True, java_version)
    )

    if JAVA_PARAM_JAVAFX_MODULES in ctx.payload.params:
        modules = ctx.payload.params[JAVA_PARAM_JAVAFX_MODULES].split(',')
        javac_args = javac_args.with_module_path('$JAVAFX_PATH/lib').with_modules(modules)
        java_args = java_args.with_module_path('$JAVAFX_PATH/lib').with_modules(modules)

    return java_args.build(), javac_args.build()
