from typing import Optional

from eva.pltform.evaluation.core import EvaluationContext, Evaluator, StyleHandler, StyleResult
from eva.pltform.evaluation.java.checkstyle import CheckstyleParser
from eva.pltform.evaluation.java.params import (
    JAVA_PARAM_SPOTBUGS_EFFORT,
    JAVA_PARAM_SPOTBUGS_MODE,
    JAVA_PARAM_STYLE_TOOL,
)
from eva.pltform.evaluation.java.spotbugs import SpotBugsParser
from eva.pltform.evaluation.java.util import to_fqcn
from eva.pltform.format.stex import StyleReport
from eva.pltform.rt import CODE_TIMEOUT


def create_style_handler(ctx: EvaluationContext, javac_args, java_args):
    tool = ctx.payload.params.get(JAVA_PARAM_STYLE_TOOL, 'checkstyle')

    if tool == 'checkstyle':
        return JavaCheckstyleStyleHandler(java_args=java_args)
    elif tool == 'spotbugs':
        return JavaSpotBugsStyleHandler(java_args=java_args, javac_args=javac_args)
    else:
        raise RuntimeError(f'style tool {tool} is not supported by evaluator')


class JavaCheckstyleStyleHandler(StyleHandler):

    def __init__(self, java_args):
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> Optional[StyleResult]:
        r = ctx.env.bash(
            [
                f'java {self.java_args} -jar $CHECKSTYLE_PATH -c sun_checks.xml -f xml',
                '-o style_report.xml .',
            ],
            timeout=ctx.opts.style_timeout,
        )
        r.expect(status_not=[CODE_TIMEOUT])

        # checkstyle exits with -2 on parsing errors
        # bash uses mod 256 on exit codes
        if r.retcode in (-2, 254):
            ctx.logger.warning(
                'checkstyle returned status code -2, probably due to an unparsable solution.'
            )
            return None

        # if parsing was successful, checkstyle exits with the number of style errors.
        # access the xml file only if parsing was successful,
        # otherwise style_report.xml may be empty or truncated (behavior seems to be undefined)

        xml = ctx.env.read_file('style_report.xml')
        parser = CheckstyleParser()
        report = parser.parse(xml)
        return StyleResult(errors=len(report.errors), report=report)


class JavaSpotBugsStyleHandler(StyleHandler):

    def __init__(self, java_args, javac_args):
        self.java_args = java_args
        self.javac_args = javac_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> Optional[StyleResult]:
        analyze_files = ctx.payload.include_files + ctx.payload.solution_files

        r = ctx.env.bash(
            [
                f'javac {self.javac_args} -cp $CLASSPATH:$EJUNIT_RUNNER_PATH',
                f'-d classes {" ".join([f.path for f in analyze_files])}',
            ],
            timeout=ctx.opts.compile_timeout,
        )
        r.expect(status_not=[CODE_TIMEOUT])

        compilation_failed = r.retcode != 0
        if compilation_failed:
            return StyleResult(errors=0, report=StyleReport(tool='', version='', errors=[]))

        cmd = [f'java {self.java_args} -jar $SPOTBUGS_PATH -textui -xml=spotbugs.xml -nested:false']
        cmd += [f'-onlyAnalyze {",".join([to_fqcn(f) for f in analyze_files])}']

        if JAVA_PARAM_SPOTBUGS_MODE in ctx.payload.params:
            cmd += [f'-{ctx.payload.params[JAVA_PARAM_SPOTBUGS_MODE]}']

        if JAVA_PARAM_SPOTBUGS_EFFORT in ctx.payload.params:
            cmd += [f'-effort:{ctx.payload.params[JAVA_PARAM_SPOTBUGS_EFFORT]}']

        cmd += ['.']

        r = ctx.env.bash(cmd, timeout=ctx.opts.style_timeout)
        r.expect(status=[0])

        xml = ctx.env.read_file('spotbugs.xml')
        parser = SpotBugsParser()
        report = parser.parse(xml)
        return StyleResult(errors=len(report.errors), report=report)
