from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.evaluation.java.params import (
    JAVA_PARAM_COVERAGE_ENABLED,
    JAVA_PARAM_COVERAGE_LINE_TYPE,
    JAVA_PARAM_COVERAGE_TOOL,
)
from eva.pltform.evaluation.java.util import to_fqcn
from eva.pltform.format.cexf import CEXFParser
from eva.pltform.format.trex import TREXParser
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


def create_test_handler(ctx: EvaluationContext, javac_args, java_args):
    coverage = ctx.payload.params.get(JAVA_PARAM_COVERAGE_ENABLED, 'false').lower() == 'true'
    if coverage:
        tool = ctx.payload.params.get(JAVA_PARAM_COVERAGE_TOOL, 'undefined')

        if tool == 'jacoco':
            return JavaJacocoTestHandler(java_args=java_args)
        elif tool == 'clover':
            return JavaCloverTestHandler(java_args=java_args, javac_args=javac_args)
        else:
            raise RuntimeError(f'coverage tool {tool} is not supported by evaluator')
    else:
        return JavaTestHandler(java_args=java_args)


class JavaTestHandler(TestHandler):
    trex = TREXParser()

    def __init__(self, java_args):
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        test_fqcns = [to_fqcn(f) for f in ctx.payload.test_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $EJUNIT_RUNNER_PATH:$CLASSPATH:classes',
                'de.hsrm.sls.ejunit.mono.Runner -o trex_report.xml',
                f'-s {ctx.opts.max_output_size}',
                *test_fqcns,
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.test_timeout,
        )
        r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])

        testsuite = self.trex.parse(ctx.env.read_file('trex_report.xml')).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=r.out())


class JavaJacocoTestHandler(TestHandler):
    trex = TREXParser()
    cexf = CEXFParser()

    def __init__(self, java_args):
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        test_fqcns = [to_fqcn(f) for f in ctx.payload.test_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $EJUNIT_RUNNER_PATH:$CLASSPATH:classes',
                '-javaagent:$JACOCOAGENT_PATH de.hsrm.sls.ejunit.mono.Runner',
                f'-o trex_report.xml -s {ctx.opts.max_output_size} -jacoco',
                *test_fqcns,
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.codecov_timeout,
        )
        r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])
        runtime_output = r.out()

        line_type = ctx.payload.params.get(JAVA_PARAM_COVERAGE_LINE_TYPE, 'line')
        solution_fqcns = [to_fqcn(f) for f in ctx.payload.solution_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $JACOCO_REPORT_PATH:$CLASSPATH:classes',
                'de.hsrm.sls.cexf.jacoco.CEXFReport -o cexf_report.xml',
                f"-ff {','.join(solution_fqcns)} -lt {line_type} .",
            ]
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output += r.out()

        testsuite = self.trex.parse(ctx.env.read_file('trex_report.xml')).fix_timeouts(
            ctx.opts.codecov_timeout
        )
        coverage = self.cexf.parse(ctx.env.read_file('cexf_report.xml')).reorder_by(testsuite)

        return self.mapper.map(testsuite=testsuite, output=runtime_output, coverage=coverage)


class JavaCloverTestHandler(TestHandler):
    trex = TREXParser()
    cexf = CEXFParser()

    INITSTR = 'build/cli/db/clover.db'
    CLASSES = 'build/cli/classes'
    INSTRDIR = 'build/cli/instr/'

    def __init__(self, javac_args, java_args):
        self.javac_args = javac_args
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $CLOVER_PATH',
                f'com.atlassian.clover.CloverInstr -i {self.INITSTR}',
                '`find . -type f -name "*.java"`',
                f'-d {self.INSTRDIR} --flushpolicy threaded --flushinterval 2000',
            ]
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output = r.out()

        ctx.env.mkdir('classes', recursive=True)

        r = ctx.env.bash(
            [
                f'javac {self.javac_args} -cp $CLASSPATH:$EJUNIT_RUNNER_PATH:$CLOVER_PATH',
                f'-d {self.CLASSES} `find {self.INSTRDIR} -type f -name "*.java"`',
            ]
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output += r.out()

        test_fqcns = [to_fqcn(f) for f in ctx.payload.test_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args}',
                f'-cp $EJUNIT_RUNNER_PATH:$CLASSPATH:$CLOVER_PATH:{self.CLASSES}',
                'de.hsrm.sls.ejunit.mono.Runner -o trex_report.xml',
                f'-s {ctx.opts.max_output_size}',
                *test_fqcns,
            ],
            max_output_size=ctx.opts.max_output_size,
            timeout=ctx.opts.codecov_timeout,
        )
        r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])
        runtime_output += r.out()

        line_type = ctx.payload.params.get(JAVA_PARAM_COVERAGE_LINE_TYPE, 'line')
        solution_fqcns = [to_fqcn(f) for f in ctx.payload.solution_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $CLOVER_PATH',
                f'de.hsrm.sls.cexf.openclover.CEXFReport -i {self.INITSTR}',
                f"-o cexf_report.xml -ff {','.join(solution_fqcns)} -lt {line_type}",
            ]
        )
        r.expect(status=[CODE_SUCCESS])

        testsuite = self.trex.parse(ctx.env.read_file('trex_report.xml')).fix_timeouts(
            ctx.opts.codecov_timeout
        )
        coverage = self.cexf.parse(ctx.env.read_file('cexf_report.xml')).reorder_by(testsuite)

        return self.mapper.map(testsuite=testsuite, output=runtime_output, coverage=coverage)
