from eva.pltform.evaluation.core import EvaluationContext, Evaluator, TestHandler, TestResult
from eva.pltform.evaluation.java.params import (
    JAVA_PARAM_COVERAGE_ENABLED,
    JAVA_PARAM_COVERAGE_LINE_TYPE,
    JAVA_PARAM_COVERAGE_TOOL,
)
from eva.pltform.evaluation.java.util import to_fqcn
from eva.pltform.evaluation.javaie.scripts import get_testcases
from eva.pltform.format.cexf import CEXFParser
from eva.pltform.format.trex import Testsuite, TREXParser
from eva.pltform.rt import CODE_SUCCESS, CODE_TIMEOUT


def create_test_handler(ctx: EvaluationContext, javac_args, java_args):
    coverage = ctx.payload.params.get(JAVA_PARAM_COVERAGE_ENABLED, 'false').lower() == 'true'
    if coverage:
        tool = ctx.payload.params.get(JAVA_PARAM_COVERAGE_TOOL, 'undefined')

        if tool == 'jacoco':
            return JavaIEJacocoTestHandler(java_args=java_args)
        elif tool == 'clover':
            return JavaIECloverTestHandler(java_args=java_args, javac_args=javac_args)
        else:
            raise RuntimeError(f'coverage tool {tool} is not supported by evaluator')
    else:
        return JavaIETestHandler(java_args=java_args)


class JavaIETestHandler(TestHandler):
    trex = TREXParser()

    def __init__(self, java_args):
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        tests = get_testcases(ctx, java_args=self.java_args)

        runtime_output = ''
        testcases = []
        for t in tests:
            r = ctx.env.bash(
                [
                    f'java {self.java_args} -cp $EJUNIT_RUNNER_PATH:$CLASSPATH:classes',
                    'de.hsrm.sls.ejunit.mono.Runner -o trex_report.xml',
                    f'-s {ctx.opts.max_output_size}',
                    t,
                ],
                max_output_size=ctx.opts.max_output_size,
                timeout=ctx.opts.test_timeout,
            )
            r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])

            runtime_output += r.out()
            testsuite = self.trex.parse(ctx.env.read_file('trex_report.xml'))
            testcases.append(testsuite.testcases[0])

        stdout = '\n'.join([t.stdout for t in testcases if len(t.stdout) != 0])
        stderr = '\n'.join([t.stderr for t in testcases if len(t.stderr) != 0])
        testsuite = Testsuite(testcases=testcases, stdout=stdout, stderr=stderr).fix_timeouts(
            ctx.opts.test_timeout
        )
        return self.mapper.map(testsuite=testsuite, output=runtime_output)


class JavaIEJacocoTestHandler(TestHandler):
    trex = TREXParser()
    cexf = CEXFParser()

    def __init__(self, java_args):
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        tests = get_testcases(ctx, java_args=self.java_args)
        runtime_output = ''
        testcases = []
        for t in tests:
            r = ctx.env.bash(
                [
                    f'java {self.java_args} -cp $EJUNIT_RUNNER_PATH:$CLASSPATH:classes',
                    '-javaagent:$JACOCOAGENT_PATH de.hsrm.sls.ejunit.mono.Runner',
                    '-o trex_report.xml -jacoco',
                    t,
                ],
                max_output_size=ctx.opts.max_output_size,
                timeout=ctx.opts.codecov_timeout,
            )
            r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])
            runtime_output += r.out()

            xml = ctx.env.read_file('trex_report.xml')
            testsuite = TREXParser().parse(xml)
            testcases.append(testsuite.testcases[0])

        line_type = ctx.payload.params.get(JAVA_PARAM_COVERAGE_LINE_TYPE, 'line')
        solution_fqcns = [to_fqcn(f) for f in ctx.payload.solution_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $JACOCO_REPORT_PATH:$CLASSPATH:classes',
                'de.hsrm.sls.cexf.jacoco.CEXFReport -o cexf_report.xml',
                f"-ff {','.join(solution_fqcns)} -lt {line_type} .",
            ],
            max_output_size=ctx.opts.max_output_size,
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output += r.out()

        testsuite = Testsuite(testcases=testcases).fix_timeouts(ctx.opts.codecov_timeout)
        coverage = self.cexf.parse(ctx.env.read_file('cexf_report.xml')).reorder_by(testsuite)

        return self.mapper.map(testsuite=testsuite, output=runtime_output, coverage=coverage)


class JavaIECloverTestHandler(TestHandler):
    trex = TREXParser()
    cexf = CEXFParser()

    INITSTR = 'build/cli/db/clover.db'
    CLASSES = 'build/cli/classes'
    INSTRDIR = 'build/cli/instr/'

    def __init__(self, javac_args, java_args):
        self.javac_args = javac_args
        self.java_args = java_args

    def run(self, evaluator: Evaluator, ctx: EvaluationContext) -> TestResult:
        tests = get_testcases(ctx, java_args=self.java_args)

        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $CLOVER_PATH',
                f'com.atlassian.clover.CloverInstr -i {self.INITSTR}',
                '`find . -type f -name "*.java"`',
                f'-d {self.INSTRDIR} --flushpolicy threaded --flushinterval 2000',
            ],
            max_output_size=ctx.opts.max_output_size,
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output = r.out()

        ctx.env.mkdir('classes', recursive=True)

        ctx.env.bash(
            [
                f'javac {self.javac_args} -cp $CLASSPATH:$EJUNIT_RUNNER_PATH:$CLOVER_PATH',
                f'-d {self.CLASSES} `find {self.INSTRDIR} -type f -name "*.java"`',
            ]
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output += r.out()

        testcases = []
        for t in tests:
            r = ctx.env.bash(
                [
                    f'java {self.java_args}',
                    f'-cp $EJUNIT_RUNNER_PATH:$CLASSPATH:$CLOVER_PATH:{self.CLASSES}',
                    'de.hsrm.sls.ejunit.mono.Runner -o trex_report.xml',
                    t,
                ],
                max_output_size=ctx.opts.max_output_size,
                timeout=ctx.opts.codecov_timeout,
            )
            r.expect(status=[CODE_SUCCESS, CODE_TIMEOUT])
            runtime_output += r.out()

            xml = ctx.env.read_file('trex_report.xml')
            testsuite = TREXParser().parse(xml)
            testcases.append(testsuite.testcases[0])

        line_type = (
            ctx.payload.params[JAVA_PARAM_COVERAGE_LINE_TYPE]
            if JAVA_PARAM_COVERAGE_LINE_TYPE in ctx.payload.params
            else 'line'
        )
        solution_fqcns = [to_fqcn(f) for f in ctx.payload.solution_files]
        r = ctx.env.bash(
            [
                f'java {self.java_args} -cp $CLOVER_PATH',
                f'de.hsrm.sls.cexf.openclover.CEXFReport -i {self.INITSTR}',
                f"-o cexf_report.xml -ff {','.join(solution_fqcns)} -lt {line_type}",
            ],
            max_output_size=ctx.opts.max_output_size,
        )
        r.expect(status=[CODE_SUCCESS])
        runtime_output += r.out()

        testsuite = Testsuite(testcases=testcases).fix_timeouts(ctx.opts.codecov_timeout)
        coverage = self.cexf.parse(ctx.env.read_file('cexf_report.xml')).reorder_by(testsuite)

        return self.mapper.map(testsuite=testsuite, output=runtime_output, coverage=coverage)
