from enum import IntEnum
from typing import List

from eva.pltform.format import trex
from lxml import etree

# pylint: disable=no-name-in-module
from lxml.etree import LxmlError
from pydantic import BaseModel


class LineDecision(IntEnum):
    NONE = 0
    MISSED = 1
    PARTIAL = 2
    FULL = 3


class LineType(IntEnum):
    LINE = 0
    INSTR = 1
    STMT = 2
    METHOD = 3
    COND = 4
    CLAZZ = 5


class Line(BaseModel):
    nr: int
    missed: int
    covered: int
    type: LineType
    decision: LineDecision


class File(BaseModel):
    name: str
    fqcn: str
    lines: List[Line]


class Testcase(BaseModel):
    id: str
    files: List[File]


class Testsuite(BaseModel):
    testcases: List[Testcase]


class Coverage(BaseModel):
    tool: str
    version: str
    testsuite: Testsuite

    def reorder_by(self, testsuite: trex.Testsuite):
        # the order of testcases in test results may not match the testcase order in coverage
        # results. this is dependent on the coverage tool, so we reorder the coverage results to
        # match the original testcase order
        testcase_ids = [t.id for t in testsuite.testcases]
        order = {testcase_id: i for i, testcase_id in enumerate(testcase_ids)}
        self.testsuite.testcases = sorted(self.testsuite.testcases, key=lambda x: order[x.id])
        return self


def get_coverage_decision(missed, covered):
    if missed != 0 and covered == 0:
        return LineDecision.MISSED

    if missed == 0 and covered != 0:
        return LineDecision.FULL

    if missed != 0 and covered != 0:
        return LineDecision.PARTIAL

    return LineDecision.NONE


def get_line_type(ltype: str):
    if ltype == 'line':
        return LineType.LINE
    elif ltype == 'instr':
        return LineType.INSTR
    elif ltype == 'stmt':
        return LineType.STMT
    elif ltype == 'method':
        return LineType.METHOD
    elif ltype == 'cond':
        return LineType.COND
    elif ltype == 'class':
        return LineType.COND
    raise RuntimeError(f'unknown CEXF line type: {ltype}')


class CEXFParser:
    def parse(self, xml):
        try:
            testcases = []
            root = etree.fromstring(xml.strip().replace('encoding="UTF-8"', ''))
            tool = root.attrib.get('tool')
            version = root.attrib.get('version')

            for test_node in root.xpath('testsuite/testcase'):
                files = []

                name = test_node.attrib.get('name')
                class_name = test_node.attrib.get('classname')
                testcase_id = f'{class_name}#{name}' if class_name else name

                file_nodes = test_node.xpath('file')
                for file_node in file_nodes:
                    file_name = file_node.attrib.get('name')
                    file_fqcn = file_node.attrib.get('fqcn')

                    line_nodes = file_node.xpath('line')
                    lines = []
                    for line_node in line_nodes:
                        nr = int(line_node.attrib.get('nr'))
                        missed = int(line_node.attrib.get('missed'))
                        covered = int(line_node.attrib.get('covered'))
                        ltype = get_line_type(line_node.attrib.get('type'))

                        decision = get_coverage_decision(missed, covered)
                        lines.append(
                            Line(
                                nr=nr, missed=missed, covered=covered, type=ltype, decision=decision
                            )
                        )

                    files.append(File(name=file_name, fqcn=file_fqcn, lines=lines))

                if len(files) > 0:
                    testcases.append(Testcase(id=testcase_id, files=files))

            testsuite = Testsuite(testcases=testcases)
            coverage = Coverage(tool=tool, version=version, testsuite=testsuite)
            return coverage
        except LxmlError as e:
            raise RuntimeError('CEXF file could not be parsed') from e
