import typing
import xml.etree.ElementTree as ET
from typing import List, Literal, Optional

from pydantic import BaseModel, Field


class Result(BaseModel):
    type: str


class SuccessResult(Result):
    type: Literal['success'] = 'success'


class ErrorResult(Result):
    type: Literal['error'] = 'error'
    details: str
    message: str


class FailureResult(Result):
    type: Literal['failure'] = 'failure'
    details: str
    message: str


class PendingResult(Result):
    type: Literal['pending'] = 'pending'


class StartedResult(Result):
    type: Literal['started'] = 'started'


class TimeoutResult(Result):
    type: Literal['timeout'] = 'timeout'
    timeout: int


ResultUnion = typing.Annotated[
    typing.Union[
        SuccessResult, ErrorResult, FailureResult, PendingResult, StartedResult, TimeoutResult
    ],
    Field(discriminator='type'),
]


class Testcase(BaseModel):
    id: str
    name: str
    class_name: typing.Optional[str]  # C has no classes
    result: ResultUnion
    stdout: typing.Optional[str]
    stderr: typing.Optional[str]


class Testsuite(BaseModel):
    skipped: int = 0
    timestamp: Optional[str] = None
    time: Optional[str] = None

    allocated: Optional[int] = None
    free: Optional[int] = None

    stdout: Optional[str] = None
    stderr: Optional[str] = None
    testcases: List[Testcase]

    def errors(self):
        return len([t for t in self.testcases if isinstance(t.result, ErrorResult)])

    def failures(self):
        return len([t for t in self.testcases if isinstance(t.result, FailureResult)])

    def passed(self):
        return len([t for t in self.testcases if isinstance(t.result, SuccessResult)])

    def failed(self):
        return len(self.testcases) - self.passed()

    def pending(self):
        return len([t for t in self.testcases if isinstance(t.result, PendingResult)])

    def timeouts(self):
        return len([t for t in self.testcases if isinstance(t.result, TimeoutResult)])

    def leaks(self):
        return (
            self.allocated - self.free
            if self.allocated is not None and self.free is not None
            else None
        )

    def correctness(self):
        total = len(self.testcases)
        return (total - self.failed()) / total if total != 0 else 0

    def finished_testcases(self):
        return [
            t
            for t in self.testcases
            if isinstance(t.result, (SuccessResult, FailureResult, ErrorResult))
        ]

    def fix_timeouts(self, timeout: int):
        # we assume that the testcase that is in state 'started' has caused the timeout
        for testcase in self.testcases:
            if isinstance(testcase.result, StartedResult):
                testcase.result = TimeoutResult(timeout=timeout)

        return self


class TREXParser:

    def parse_runtime_error(self, ele):
        if ele is None:
            return None

        details = ''.join(ele.itertext()).strip()
        message = ele.get('message') or ''
        # self.type = ele.get('type') or ''
        return ErrorResult(details=details, message=message)

    def parse_assertion_error(self, ele):
        if ele is None:
            return None

        details = ''.join(ele.itertext()).strip()
        message = ele.get('message') or ''
        return FailureResult(details=details, message=message)

    def parse_timeout(self, ele):
        if ele is None:
            return None

        timeout = int(ele.get('timeout'))
        return TimeoutResult(timeout=timeout)

    def parse_testcase(self, ele):
        name = ele.get('name')
        class_name = ele.get('classname')
        # self.time = ele.get('time')

        failure_ele = ele.find('failure')
        error_ele = ele.find('error')
        pending_ele = ele.find('pending')
        started_ele = ele.find('started')
        timeout_ele = ele.find('timeout')

        if failure_ele is not None:
            result = self.parse_assertion_error(failure_ele)
        elif error_ele is not None:
            result = self.parse_runtime_error(error_ele)
        elif pending_ele is not None:
            result = PendingResult()
        elif started_ele is not None:
            result = StartedResult()
        elif timeout_ele is not None:
            result = self.parse_timeout(timeout_ele)
        else:
            result = SuccessResult()

        stdout_ele = ele.find('system-out')
        stdout = ''.join(stdout_ele.itertext()).strip() if stdout_ele is not None else None

        stderr_ele = ele.find('system-err')
        stderr = ''.join(stderr_ele.itertext()).strip() if stderr_ele is not None else None

        testcase_id = f'{class_name}#{name}' if class_name else name
        return Testcase(
            id=testcase_id,
            name=name,
            class_name=class_name,
            result=result,
            stdout=stdout,
            stderr=stderr,
        )

    def parse(self, xml):
        try:
            root = ET.fromstring(xml)

            # C evaluator provides metadata about memory usage in storage element
            storage_ele = root.find('storage')
            if storage_ele is not None:
                allocated = int(storage_ele.get('allocated') or 0)
                free = int(storage_ele.get('free') or 0)
            else:
                allocated = None
                free = None

            timestamp = root.get('timestamp')
            time = root.get('time')

            stdout_ele = root.find('system-out')
            stdout = ''.join(stdout_ele.itertext()).strip() if stdout_ele is not None else None

            stderr_ele = root.find('system-err')
            stderr = ''.join(stderr_ele.itertext()).strip() if stderr_ele is not None else None

            testcase_elements = root.findall('.//testcase')

            testcases = []
            for ele in testcase_elements:
                testcase = self.parse_testcase(ele)
                testcases.append(testcase)

            return Testsuite(
                timestamp=timestamp,
                time=time,
                allocated=allocated,
                free=free,
                stdout=stdout,
                stderr=stderr,
                testcases=testcases,
            )
        except ET.ParseError as e:
            raise RuntimeError('trex file could not be parsed') from e
