import os
import tarfile
import tempfile
from dataclasses import dataclass

from docker.models.containers import Container
from eva.pltform.rt.core import CODE_TIMEOUT, EnvironmentHandle, ExecResult


def truncate(text, max_size):
    return f'{text[:max_size]}... (truncated)' if len(text) >= max_size else text


@dataclass
class DockerEnvironmentHandle(EnvironmentHandle):
    container: Container
    DEFAULT_EXEC_TIMEOUT = 60

    def exec(self, cmd, timeout: int = None, max_output_size: int = None):
        self.ensure_alive()

        timeout = timeout if timeout is not None else self.DEFAULT_EXEC_TIMEOUT

        # exec_run has no timeout param and it appears that it is very hard to implement:
        # https://github.com/docker/docker-py/issues/2651
        # so we just use the timeout unix command as workaround.
        # we need to use --signal=SIGKILL because sometimes the subprocess catches SIGTERM (the
        # default), causing the command to
        # run indefinitely
        plain_cmd = ' '.join(cmd)
        proc = self.container.exec_run(
            f'/usr/bin/timeout --signal=SIGKILL {timeout} {plain_cmd}', workdir=self.cwd, demux=True
        )

        # the timeout unix command returns 137 if the subprocess was killed
        # initially we used another exit code for indicating timeouts
        retcode = proc.exit_code if proc.exit_code != 137 else CODE_TIMEOUT

        if retcode == CODE_TIMEOUT:
            self.request_logger.debug(f'timeout reached for cmd: {plain_cmd}')

        stdout, stderr = proc.output

        stdout = stdout.decode() if stdout is not None else ''
        stdout = truncate(stdout, max_output_size) if max_output_size is not None else stdout

        stderr = stderr.decode() if stderr is not None else ''
        stderr = truncate(stderr, max_output_size) if max_output_size is not None else stderr

        self.request_logger.debug(f'{cmd} stdout: {stdout}')
        self.request_logger.debug(f'{cmd} stderr: {stderr}')

        return ExecResult(cmd=plain_cmd, retcode=retcode, stdout=stdout, stderr=stderr)

    def create_file(self, content: str, path: str, **kwargs):
        self.ensure_alive()

        dir_path = os.path.dirname(os.path.abspath(path))
        self.mkdir(dir_path, recursive=True)

        src_file_name = os.path.basename(path)
        with tempfile.TemporaryDirectory() as temp_dir:
            src_path = os.path.join(temp_dir, src_file_name)
            with open(src_path, 'w', encoding='utf-8') as f:
                f.write(content)

            with tarfile.open(src_path + '.tar', mode='w') as tar:
                tar.add(src_path, arcname=src_file_name)

            with open(src_path + '.tar', 'rb') as f:
                data = f.read()

            self.container.put_archive(os.path.dirname(path), data)
            self.request_logger.debug(f'created file at {path}')

    # pylint: disable=pointless-string-statement
    """
    def insert_dir(self, dir_path: str, dst: str, **kwargs):
        src_file_name = os.path.basename(dst)
        temp_dir = tempfile.TemporaryDirectory()
        try:
            src_path = os.path.join(temp_dir.name, src_file_name)
            tar = tarfile.open(src_path + '.tar', mode='w')
            try:
                tar.add(dir_path, arcname=src_file_name)
            finally:
                tar.close()

            data = open(src_path + '.tar', 'rb').read()
            self.container.put_archive(os.path.dirname(dst), data)
        finally:
            temp_dir.cleanup()
    """
