import contextlib
import math
import threading
from datetime import datetime

import dateutil.parser
import docker
import loguru
from cachetools import TTLCache
from docker import DockerClient
from docker.models.images import Image
from eva.pltform.rt.core import EnvironmentHandle, Runtime, RuntimeInterruptedError
from eva.pltform.rt.dockert.handle import DockerEnvironmentHandle


class DockerImageProvider:
    def __init__(self, base_path, registry, creds, ttl):
        self.base_path = base_path
        self.registry = registry
        self.creds = creds
        self.cache = TTLCache(maxsize=math.inf, ttl=ttl)
        self.lock = threading.Lock()

    def get(
        self, environment_id: str, client: DockerClient, request_logger: 'loguru.Logger'
    ) -> Image:
        with self.lock:
            if self.registry is None:
                return client.images.get(f'{self.base_path}/{environment_id}:latest')
            else:
                if self.creds is not None:
                    user, password = self.creds
                    client.login(username=user, password=password, registry=self.registry)

                image_path = f'{self.registry}/{self.base_path}/{environment_id}:latest'

                if image_path not in self.cache:
                    return self._get_latest(client, image_path, request_logger)
                else:
                    try:
                        return client.images.get(image_path)
                    except docker.errors.ImageNotFound:
                        return self._pull(client, image_path, request_logger)

    def _get_latest(self, client, image_path: str, request_logger: 'loguru.Logger') -> Image:
        try:
            local = client.images.get(image_path)

            try:
                m = client.images.get_registry_data(image_path)
                remote = m.pull()
            except docker.errors.APIError:
                # may be a "dial tcp ....: i/o timeout" because container registry is unreachable
                request_logger.exception(
                    f'Failed to pull image {image_path}. Using local version...'
                )
                return local

            if self._last_tag_time(local) is not None and self._last_tag_time(
                local
            ) > self._created(remote):
                return local

            return self._pull(client, image_path, request_logger)
        except docker.errors.ImageNotFound:
            return self._pull(client, image_path, request_logger)

    def _last_tag_time(self, image: Image) -> datetime:
        tt = dateutil.parser.isoparse(image.attrs.get('Metadata').get('LastTagTime'))
        is_default = tt.month == 1 and tt.day == 1 and tt.year == 1
        return None if is_default else tt

    def _created(self, image: Image) -> datetime:
        return dateutil.parser.isoparse(image.attrs.get('Created'))

    def _pull(self, client, image_path: str, request_logger: 'loguru.Logger') -> Image:
        request_logger.info(f'Pulling image {image_path}.')
        image = client.images.pull(image_path)
        request_logger.info(f'Image {image_path} pulled')
        self.cache[image_path] = True
        return image


class DockerRuntimeDefaults:
    MEM_LIMIT = '512m'
    CPU_QUOTA = 1
    MAX_INSTANCES = 4
    AUTO_SHUTDOWN = 600  # in seconds
    WORKDIR = '/test'
    NETWORK_MODE = 'bridge'  # host, bridge, none
    IMG_CACHE_TTL = 60  # in seconds, -1 for infinity
    IMG_REGISTRY = None
    IMG_REGISTRY_USER = None
    IMG_REGISTRY_PASSWORD = None
    IMG_BASE_PATH = 'eva/environments'


class DockerRuntime(Runtime):
    def __init__(
        self,
        max_instances=DockerRuntimeDefaults.MAX_INSTANCES,
        mem_limit=DockerRuntimeDefaults.MEM_LIMIT,
        cpu_quota=DockerRuntimeDefaults.CPU_QUOTA,
        workdir=DockerRuntimeDefaults.WORKDIR,
        auto_shutdown=DockerRuntimeDefaults.AUTO_SHUTDOWN,
        network_mode=DockerRuntimeDefaults.NETWORK_MODE,
        img_cache_ttl=DockerRuntimeDefaults.IMG_CACHE_TTL,
        img_base_path=DockerRuntimeDefaults.IMG_BASE_PATH,
        img_registry=DockerRuntimeDefaults.IMG_REGISTRY,
        img_registry_user=DockerRuntimeDefaults.IMG_REGISTRY_USER,
        img_registry_password=DockerRuntimeDefaults.IMG_REGISTRY_PASSWORD,
    ):
        super().__init__()
        self.semaphore = threading.Semaphore(max_instances)
        self.mem_limit = mem_limit
        self.cpu_quota = cpu_quota
        self.image_provider = DockerImageProvider(
            registry=img_registry,
            creds=(
                (img_registry_user, img_registry_password)
                if img_registry_user and img_registry_password
                else None
            ),
            base_path=img_base_path,
            ttl=img_cache_ttl if img_cache_ttl != -1 else math.inf,
        )
        self.workdir = workdir
        self.auto_shutdown = auto_shutdown
        self.network_mode = network_mode
        self.handles = []

    @contextlib.contextmanager
    def create(self, environment_id: str, request_logger: 'loguru.Logger') -> EnvironmentHandle:
        with self.semaphore:
            if self.stopped.is_set():
                raise RuntimeInterruptedError()

            client = docker.from_env()

            request_logger.debug(f'Preparing docker container for environment {environment_id}')
            image = self.image_provider.get(environment_id, client, request_logger)
            request_logger.debug(f'Using local image {image.short_id}.')

            container = client.containers.run(
                image.id,
                # if the python script that manages the runtime is killed with SIGKILL,
                # started containers
                # are never stopped and stay alive until the docker daemon is restarted. We use
                # this command to ensure that
                # orphan containers are removed automatically after some time.
                # (a container stops if the entry process finishes)
                command=f'/bin/bash -c "sleep {self.auto_shutdown} || exit 0"',
                # default network_mode is bridge, which creates a bridge interface for each
                # container.
                # that will slow down execution in the container by 50% (!!!). This observation
                # is in line with an older study:
                # https://stackoverflow.com/questions/21889053/what-is-the-runtime-performance
                # -cost-of-a-docker-container
                # Note that network_mode='host' also reduces performance (but has significantly
                # less impact, about 10-15%) and should
                # be used only if the containers need network connections. in bulk evaluation
                # scenarios the network_mode should
                # be set to 'none' to achieve a native-like performance
                network_mode=self.network_mode,
                # for some reason this seems to be necessary to get native-like performance:
                network_disabled=self.network_mode == 'none',
                detach=True,
                remove=True,
                mem_limit=self.mem_limit,
                nano_cpus=int(self.cpu_quota * 10**9),
            )

            try:
                env = DockerEnvironmentHandle(
                    environment_id=environment_id,
                    container=container,
                    stopped=self.stopped,
                    cwd='/',
                    request_logger=request_logger,
                )
                env.mkdir(self.workdir, recursive=True)
                env.cwd = self.workdir

                self.handles.append(env)
                yield env
            finally:
                container.kill()
                self.handles.remove(env)
                client.close()

    def describe(self) -> dict[str, str]:
        return {
            'type': 'docker',
            'mem_limit': self.mem_limit,
            'cpu_quota': self.cpu_quota,
            'auto_shutdown': self.auto_shutdown,
            'network_mode': self.network_mode,
        }
