import os
import subprocess

import pytest
from _pytest.logging import LogCaptureFixture
from eva.pltform import EvaluatorRegistry
from eva.pltform.evaluation.core import EvaluatorOptions
from eva.pltform.rt.dockert import DockerRuntime
from loguru import logger
from tests.integration.util.context import TestContext
from tests.integration.util.logging import setup_logging
from tests.util import FixtureLoader

pytest.register_assert_rewrite('tests.integration.util.asserts')

ARG_LOG_LEVEL = '--log_level'
ARG_BUILD = '--build'
ARG_ENV = '--env'


@pytest.fixture(scope='session', name='runtime')
def fixture_runtime():
    return DockerRuntime(max_instances=1)


@pytest.fixture(scope='session', name='fixtures')
def fixture_fixtures():
    script_dir = os.path.dirname(os.path.abspath(__file__))
    return FixtureLoader(os.path.join(script_dir, 'fixtures'))


@pytest.fixture(scope='session', name='evaluators')
def fixture_evaluators():
    return EvaluatorRegistry()


@pytest.fixture
def ctx(evaluator_id, environment_id, runtime, fixtures, evaluators):
    opts = EvaluatorOptions(
        compile_timeout=10,
        style_timeout=10,
        test_timeout=20,
        codecov_timeout=30,
    )
    evaluator_cls = evaluators.find_by_id(evaluator_id)
    return TestContext(
        environment_id=environment_id,
        runtime=runtime,
        evaluator=evaluator_cls(),
        opts=opts,
        request_logger=logger,
        fixtures=fixtures,
    )


def build_evaluator(evaluators_dir, evaluator_dir):
    process = subprocess.run(
        ['bash', os.path.join(evaluators_dir, 'build.sh'), evaluator_dir],
        capture_output=True,
        text=True,
        check=False,
        cwd=evaluators_dir,
    )
    return process.returncode, process.stdout, process.stderr


def pytest_collection_modifyitems(config, items):
    arg = config.getoption(ARG_ENV)
    if arg is None:
        return

    envs = arg.split(',')

    skip = pytest.mark.skip(reason=f'does not match environment {arg}')
    for item in items:
        if not hasattr(item, 'callspec'):
            continue

        environment_id = item.callspec.params['environment_id']
        if environment_id not in envs:
            item.add_marker(skip)


def pytest_collection_finish(session):
    setup_logging(session.config.getoption(ARG_LOG_LEVEL))

    if not session.config.getoption(ARG_BUILD):
        return

    items = [
        i
        for i in session.items
        if i.get_closest_marker('evaluator') is not None and not i.get_closest_marker('skip')
    ]
    envs = {i.callspec.params['environment_id'] for i in items}

    script_dir = os.path.dirname(os.path.abspath(__file__))
    evaluators_dir = os.path.join(script_dir, '../../../environments')

    for env in envs:
        logger.info(f'building {env}')
        ret, stdout, stderr = build_evaluator(evaluators_dir, env)
        if ret != 0:
            logger.error(f'build failed for env {env}')
            logger.info(stdout)
            logger.error(stderr)


def pytest_generate_tests(metafunc):
    marker = metafunc.definition.get_closest_marker('evaluator')

    env_map = {
        'java': ['java19', 'java20', 'java21'],
        'javaie': ['java19'],
        'c': ['c17'],
        'mariadb': ['mariadb10'],
        'cwithjava': ['cwithjava'],
        'cpp': ['cpp13'],
        'python': ['python3.9', 'python3.12'],
        'js': ['js'],
        'go': ['go1.20'],
        'evarela': ['evarela'],
        'turing': ['turing'],
        'hs': ['hs'],
        'scala': ['scala'],
        'elm': ['elm'],
        'kotlin': ['kotlin1.9-java20'],
    }

    if marker is not None:
        evaluator_id = marker.args[0]
        environments = env_map.get(evaluator_id)

        metafunc.parametrize('evaluator_id', [evaluator_id])
        metafunc.parametrize('environment_id', environments)


@pytest.fixture
def caplog(_caplog: LogCaptureFixture):
    handler_id = logger.add(
        _caplog.handler,
        format='{message}',
        level=0,
        filter=lambda record: record['level'].no >= _caplog.handler.level,
        enqueue=False,  # Set to 'True' if your test is spawning child processes.
    )
    yield _caplog
    logger.remove(handler_id)
