import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('c')
class CTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='beispiele.c', content=ctx.fixtures.load('c/passing/beispiele.c'))
            ],
            test_files=[EvaFile(path='test.c', content=ctx.fixtures.load('c/passing/test.c'))],
            include_files=[
                EvaFile(path='beispiele.h', content=ctx.fixtures.load('c/passing/beispiele.h'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='beispiele.c', content=ctx.fixtures.load('c/compfailed/beispiele.c'))
            ],
            test_files=[EvaFile(path='test.c', content=ctx.fixtures.load('c/compfailed/test.c'))],
            include_files=[
                EvaFile(path='beispiele.h', content=ctx.fixtures.load('c/compfailed/beispiele.h'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert (
            './beispiele.c:4:3: error: continue statement not within a loop'
            in result.compilation.output
        )
        assert result.test is None

    def test_main(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='beispiele.c', content=ctx.fixtures.load('c/main/beispiele.c'))
            ],
            test_files=[EvaFile(path='test.c', content=ctx.fixtures.load('c/main/test.c'))],
            include_files=[
                EvaFile(path='beispiele.h', content=ctx.fixtures.load('c/main/beispiele.h'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert 'main-Funktion' in result.compilation.output
        assert result.test is None

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='beispiele.c', content=ctx.fixtures.load('c/failure/beispiele.c'))
            ],
            test_files=[EvaFile(path='test.c', content=ctx.fixtures.load('c/failure/test.c'))],
            include_files=[
                EvaFile(path='beispiele.h', content=ctx.fixtures.load('c/failure/beispiele.h'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness, 0.01) == 0.33
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 2
        assert result.test.passed == 1
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 2
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[1].result, FailureResult)
        assert isinstance(result.test.testsuite.testcases[2].result, FailureResult)

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='beispiele.c', content=ctx.fixtures.load('c/infloop/beispiele.c'))
            ],
            test_files=[EvaFile(path='test.c', content=ctx.fixtures.load('c/infloop/test.c'))],
            include_files=[
                EvaFile(path='beispiele.h', content=ctx.fixtures.load('c/infloop/beispiele.h'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 2

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='beispiele.c', content=ctx.fixtures.load('c/stackoverflow/beispiele.c')
                )
            ],
            test_files=[
                EvaFile(path='test.c', content=ctx.fixtures.load('c/stackoverflow/test.c'))
            ],
            include_files=[
                EvaFile(
                    path='beispiele.h', content=ctx.fixtures.load('c/stackoverflow/beispiele.h')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 2
