import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('cpp')
class CppTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/passing/fac.cc'))
            ],
            include_files=[EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/passing/fac.h'))],
            test_files=[
                EvaFile(path='fac_test.cc', content=ctx.fixtures.load('cpp/passing/fac_test.cc'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/failure/fac.cc'))
            ],
            include_files=[EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/failure/fac.h'))],
            test_files=[
                EvaFile(path='fac_test.cc', content=ctx.fixtures.load('cpp/failure/fac_test.cc'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness, 0.01) == 0.33
        assert len(result.test.testsuite.testcases)
        assert result.test.testcases == 3
        assert result.test.failed == 2
        assert result.test.passed == 1
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 2
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        for t in result.test.testsuite.testcases[1:]:
            assert isinstance(t.result, FailureResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/compfailed/fac.cc'))
            ],
            include_files=[
                EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/compfailed/fac.h'))
            ],
            test_files=[
                EvaFile(path='fac_test.cc', content=ctx.fixtures.load('cpp/compfailed/fac_test.cc'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert (
            '/test/fac.cc:3:10: error: unable to find numeric literal operator'
            in result.compilation.output
        )
        assert result.test is None

    def test_main(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/main/fac.cc'))],
            include_files=[EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/main/fac.h'))],
            test_files=[
                EvaFile(path='fac_test.cc', content=ctx.fixtures.load('cpp/main/fac_test.cc'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert 'main-Funktion' in result.compilation.output
        assert result.test is None

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/infloop/fac.cc'))
            ],
            include_files=[EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/infloop/fac.h'))],
            test_files=[
                EvaFile(path='fac_test.cc', content=ctx.fixtures.load('cpp/infloop/fac_test.cc'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/stackoverflow/fac.cc'))
            ],
            include_files=[
                EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/stackoverflow/fac.h'))
            ],
            test_files=[
                EvaFile(
                    path='fac_test.cc', content=ctx.fixtures.load('cpp/stackoverflow/fac_test.cc')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness, 0.01) == 0.33
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 2
        assert result.test.passed == 1
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 2
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        for t in result.test.testsuite.testcases[1:]:
            assert isinstance(t.result, FailureResult)

    def test_multitest(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/multitest/fac.cc'))
            ],
            include_files=[EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/multitest/fac.h'))],
            test_files=[
                EvaFile(
                    path='fac_test1.cc', content=ctx.fixtures.load('cpp/multitest/fac_test1.cc')
                ),
                EvaFile(
                    path='fac_test2.cc', content=ctx.fixtures.load('cpp/multitest/fac_test2.cc')
                ),
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_altinclude(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.cc', content=ctx.fixtures.load('cpp/altinclude/fac.cc'))
            ],
            include_files=[
                EvaFile(path='fac.h', content=ctx.fixtures.load('cpp/altinclude/fac.h'))
            ],
            test_files=[
                EvaFile(path='fac_test.cc', content=ctx.fixtures.load('cpp/altinclude/fac_test.cc'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)
