import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, PendingResult, SuccessResult, TimeoutResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('cwithjava')
class CWithJavaTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='ZeigerInC.c', content=ctx.fixtures.load('cwithjava/passing/ZeigerInC.c')
                )
            ],
            test_files=[
                EvaFile(path='al_test.h', content=ctx.fixtures.load('cwithjava/passing/al_test.h')),
                EvaFile(path='al_test.c', content=ctx.fixtures.load('cwithjava/passing/al_test.c')),
                EvaFile(
                    path='name/panitz/ZeigerTest.java',
                    content=ctx.fixtures.load('cwithjava/passing/ZeigerTest.java'),
                ),
            ],
            include_files=[
                EvaFile(
                    path='ZeigerInC.h', content=ctx.fixtures.load('cwithjava/passing/ZeigerInC.h')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 0
        assert result.test.passed == 2
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='ZeigerInC.c',
                    content=ctx.fixtures.load('cwithjava/compfailed/ZeigerInC.c'),
                )
            ],
            test_files=[
                EvaFile(
                    path='al_test.h', content=ctx.fixtures.load('cwithjava/compfailed/al_test.h')
                ),
                EvaFile(
                    path='al_test.c', content=ctx.fixtures.load('cwithjava/compfailed/al_test.c')
                ),
                EvaFile(
                    path='name/panitz/ZeigerTest.java',
                    content=ctx.fixtures.load('cwithjava/compfailed/ZeigerTest.java'),
                ),
            ],
            include_files=[
                EvaFile(
                    path='ZeigerInC.h',
                    content=ctx.fixtures.load('cwithjava/compfailed/ZeigerInC.h'),
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 2
        assert result.test.passed == 0
        assert (
            './ZeigerInC.c:6:3: error: continue statement not within a loop' in result.test.output
        )

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='ZeigerInC.c', content=ctx.fixtures.load('cwithjava/infloop/ZeigerInC.c')
                )
            ],
            test_files=[
                EvaFile(path='al_test.h', content=ctx.fixtures.load('cwithjava/infloop/al_test.h')),
                EvaFile(path='al_test.c', content=ctx.fixtures.load('cwithjava/infloop/al_test.c')),
                EvaFile(
                    path='name/panitz/ZeigerTest.java',
                    content=ctx.fixtures.load('cwithjava/infloop/ZeigerTest.java'),
                ),
            ],
            include_files=[
                EvaFile(
                    path='ZeigerInC.h', content=ctx.fixtures.load('cwithjava/infloop/ZeigerInC.h')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 2
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 1
        assert len(result.test.output) != 0
        assert isinstance(result.test.testsuite.testcases[0].result, TimeoutResult)
        for t in result.test.testsuite.testcases[1:]:
            assert isinstance(t.result, PendingResult)

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='ZeigerInC.c',
                    content=ctx.fixtures.load('cwithjava/stackoverflow/ZeigerInC.c'),
                )
            ],
            test_files=[
                EvaFile(
                    path='al_test.h', content=ctx.fixtures.load('cwithjava/stackoverflow/al_test.h')
                ),
                EvaFile(
                    path='al_test.c', content=ctx.fixtures.load('cwithjava/stackoverflow/al_test.c')
                ),
                EvaFile(
                    path='name/panitz/ZeigerTest.java',
                    content=ctx.fixtures.load('cwithjava/stackoverflow/ZeigerTest.java'),
                ),
            ],
            include_files=[
                EvaFile(
                    path='ZeigerInC.h',
                    content=ctx.fixtures.load('cwithjava/stackoverflow/ZeigerInC.h'),
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 2
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 2
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)
            assert t.result.message == 'Segmentation Fault'

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='ZeigerInC.c', content=ctx.fixtures.load('cwithjava/failure/ZeigerInC.c')
                )
            ],
            test_files=[
                EvaFile(path='al_test.h', content=ctx.fixtures.load('cwithjava/failure/al_test.h')),
                EvaFile(path='al_test.c', content=ctx.fixtures.load('cwithjava/failure/al_test.c')),
                EvaFile(
                    path='name/panitz/ZeigerTest.java',
                    content=ctx.fixtures.load('cwithjava/failure/ZeigerTest.java'),
                ),
            ],
            include_files=[
                EvaFile(
                    path='ZeigerInC.h', content=ctx.fixtures.load('cwithjava/failure/ZeigerInC.h')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0.5
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 1
        assert result.test.passed == 1
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 1
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[1].result, FailureResult)
