import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import ErrorResult, FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('go')
class GoTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.go', content=ctx.fixtures.load('go/passing/fac.go'))],
            test_files=[
                EvaFile(path='fac_test.go', content=ctx.fixtures.load('go/passing/fac_test.go'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.go', content=ctx.fixtures.load('go/compfailed/fac.go'))
            ],
            test_files=[
                EvaFile(path='fac_test.go', content=ctx.fixtures.load('go/compfailed/fac_test.go'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert (
            'syntax error: unexpected literal 17 at end of statement' in result.compilation.output
        )

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.go', content=ctx.fixtures.load('go/failure/fac.go'))],
            test_files=[
                EvaFile(path='fac_test.go', content=ctx.fixtures.load('go/failure/fac_test.go'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 3
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[EvaFile(path='fac.go', content=ctx.fixtures.load('go/infloop/fac.go'))],
            test_files=[
                EvaFile(path='fac_test.go', content=ctx.fixtures.load('go/infloop/fac_test.go'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.go', content=ctx.fixtures.load('go/stackoverflow/fac.go'))
            ],
            test_files=[
                EvaFile(
                    path='fac_test.go', content=ctx.fixtures.load('go/stackoverflow/fac_test.go')
                )
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 1
        assert result.test.testcases == 1
        assert result.test.failed == 1
        assert result.test.passed == 0
        assert result.test.stats.errors == 1
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, ErrorResult)
