import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.cexf import LineDecision
from eva.pltform.format.trex import SuccessResult, TimeoutResult
from tests.integration.util.asserts import assert_line_coverage
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('javaie')
class JavaIECloverTest:

    def test_multitest_coverage_clover(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='AL.java', content=ctx.fixtures.load('java/multitest/AL.java'))
            ],
            test_files=[
                EvaFile(
                    path='AL1Test.java', content=ctx.fixtures.load('java/multitest/AL1Test.java')
                ),
                EvaFile(
                    path='AL2Test.java', content=ctx.fixtures.load('java/multitest/AL2Test.java')
                ),
            ],
            compile=True,
            test=True,
            params={'coverage.enabled': 'true', 'coverage.tool': 'clover'},
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert result.test.coverage is not None
        assert result.test.coverage.tool == 'clover'
        assert result.test.testcases == 2

        assert_line_coverage(
            result.test.coverage,
            'AL1Test#testSize1',
            'AL.java',
            {
                LineDecision.FULL: [3, 4],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [8, 9],
            },
        )

        assert_line_coverage(
            result.test.coverage,
            'AL2Test#testSize2',
            'AL.java',
            {
                LineDecision.FULL: [8, 9],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [3, 4],
            },
        )

    def test_coverage_clover(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/coverage/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/coverage/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
            params={'coverage.enabled': 'true', 'coverage.tool': 'clover'},
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert result.test.coverage is not None
        assert result.test.coverage.tool == 'clover'
        assert result.test.testcases == 4

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test1',
            'Heron.java',
            {
                LineDecision.FULL: [3, 7, 11, 14, 15, 17],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [8],
            },
        )

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test2',
            'Heron.java',
            {
                LineDecision.FULL: [3, 7, 11, 14, 15, 17],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [8],
            },
        )

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test3',
            'Heron.java',
            {
                LineDecision.FULL: [3, 7, 11, 14, 15, 17],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [8],
            },
        )

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test4',
            'Heron.java',
            {
                LineDecision.FULL: [7, 8],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [3, 11, 14, 15, 17],
            },
        )

    def test_coverage_infloop_clover(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/infloop/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/infloop/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
            params={'coverage.enabled': 'true', 'coverage.tool': 'clover'},
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert result.test.coverage is not None
        assert result.test.coverage.tool == 'clover'
        assert len(result.test.testsuite.testcases) == 4
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[1].result, TimeoutResult)
        assert isinstance(result.test.testsuite.testcases[2].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[3].result, SuccessResult)

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test1',
            'Heron.java',
            {
                LineDecision.FULL: [3, 6, 7, 9],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [],
            },
        )

        assert_line_coverage(result.test.coverage, 'HeronTest#test2', 'Heron.java', None)

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test3',
            'Heron.java',
            {
                LineDecision.FULL: [3, 6, 7, 9],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [],
            },
        )

        assert_line_coverage(
            result.test.coverage,
            'HeronTest#test4',
            'Heron.java',
            {
                LineDecision.FULL: [3, 6, 7, 9],
                LineDecision.PARTIAL: [],
                LineDecision.MISSED: [],
            },
        )
