import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import FailureResult, SuccessResult, TimeoutResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('javaie')
class JavaIETestTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/passing/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/passing/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_multitest(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='AL.java', content=ctx.fixtures.load('java/multitest/AL.java'))
            ],
            test_files=[
                EvaFile(
                    path='AL1Test.java', content=ctx.fixtures.load('java/multitest/AL1Test.java')
                ),
                EvaFile(
                    path='AL2Test.java', content=ctx.fixtures.load('java/multitest/AL2Test.java')
                ),
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testsuite.testcases[0].class_name == 'AL1Test'
        assert result.test.testsuite.testcases[0].name == 'testSize1'
        assert result.test.testsuite.testcases[1].class_name == 'AL2Test'
        assert result.test.testsuite.testcases[1].name == 'testSize2'

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/infloop/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/infloop/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0.75
        assert len(result.test.testsuite.testcases) == 4
        assert result.test.testcases == 4
        assert result.test.failed == 1
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 0
        assert len(result.test.output) != 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[1].result, TimeoutResult)
        assert isinstance(result.test.testsuite.testcases[2].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[3].result, SuccessResult)

    def test_asserts_enabled(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/asserts/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/asserts/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
            params={'jvm.enable_asserts': 'true'},
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0.5
        assert len(result.test.testsuite.testcases) == 2
        assert isinstance(result.test.testsuite.testcases[0].result, FailureResult)
        assert isinstance(result.test.testsuite.testcases[1].result, SuccessResult)

    def test_asserts_disabled(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Heron.java', content=ctx.fixtures.load('java/asserts/Heron.java'))
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java', content=ctx.fixtures.load('java/asserts/HeronTest.java')
                )
            ],
            include_files=[],
            compile=True,
            test=True,
            params={'jvm.enable_asserts': 'false'},
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert isinstance(result.test.testsuite.testcases[0].result, FailureResult)
        assert isinstance(result.test.testsuite.testcases[1].result, FailureResult)

    def test_fail_dynamic_asserts(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='Heron.java', content=ctx.fixtures.load('java/dynamic_asserts/Heron.java')
                )
            ],
            test_files=[
                EvaFile(
                    path='HeronTest.java',
                    content=ctx.fixtures.load('java/dynamic_asserts/HeronTest.java'),
                )
            ],
            include_files=[],
            compile=True,
            test=True,
            params={},
        )

        with pytest.raises(Exception) as context:
            ctx.eval(payload)

        assert 'setDefaultAssertionStatus' in str(context.value)
