import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import (
    ErrorResult,
    FailureResult,
    PendingResult,
    SuccessResult,
    TimeoutResult,
)
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('kotlin')
class KotlinTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.kt', content=ctx.fixtures.load('kotlin/passing/Fac.kt'))
            ],
            test_files=[
                EvaFile(path='FacTest.kt', content=ctx.fixtures.load('kotlin/passing/FacTest.kt'))
            ],
            compile=True,
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_compfailed(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.kt', content=ctx.fixtures.load('kotlin/compfailed/Fac.kt'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.kt', content=ctx.fixtures.load('kotlin/compfailed/FacTest.kt')
                )
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert not result.compilation.compiling
        assert 'error: expecting member declaration' in result.compilation.output

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.kt', content=ctx.fixtures.load('kotlin/failure/Fac.kt'))
            ],
            test_files=[
                EvaFile(path='FacTest.kt', content=ctx.fixtures.load('kotlin/failure/FacTest.kt'))
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 3
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.kt', content=ctx.fixtures.load('kotlin/stackoverflow/Fac.kt'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.kt', content=ctx.fixtures.load('kotlin/stackoverflow/FacTest.kt')
                )
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 3
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, ErrorResult)

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='Fac.kt', content=ctx.fixtures.load('kotlin/infloop/Fac.kt'))
            ],
            test_files=[
                EvaFile(path='FacTest.kt', content=ctx.fixtures.load('kotlin/infloop/FacTest.kt'))
            ],
            compile=True,
            test=True,
        )

        result = ctx.eval(payload)
        assert result.compilation.compiling
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 1
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 1
        assert isinstance(result.test.testsuite.testcases[0].result, FailureResult)
        assert isinstance(result.test.testsuite.testcases[1].result, TimeoutResult)
        assert isinstance(result.test.testsuite.testcases[2].result, PendingResult)
