import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import (
    ErrorResult,
    FailureResult,
    PendingResult,
    SuccessResult,
    TimeoutResult,
)
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('mariadb')
class MariaDBTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql', content=ctx.fixtures.load('mariadb/passing/solution.sql')
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt', content=ctx.fixtures.load('mariadb/passing/expected.txt')
                )
            ],
            include_files=[
                EvaFile(path='import.sql', content=ctx.fixtures.load('mariadb/passing/import.sql'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 0
        assert result.test.passed == 2
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql', content=ctx.fixtures.load('mariadb/failure/solution.sql')
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt', content=ctx.fixtures.load('mariadb/failure/expected.txt')
                )
            ],
            include_files=[
                EvaFile(path='import.sql', content=ctx.fixtures.load('mariadb/failure/import.sql'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 0.5
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 1
        assert result.test.passed == 1
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 1
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[1].result, FailureResult)

    def test_sqlsyntaxerr(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql',
                    content=ctx.fixtures.load('mariadb/sqlsyntaxerr/solution.sql'),
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt',
                    content=ctx.fixtures.load('mariadb/sqlsyntaxerr/expected.txt'),
                )
            ],
            include_files=[
                EvaFile(
                    path='import.sql', content=ctx.fixtures.load('mariadb/sqlsyntaxerr/import.sql')
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 0.5
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 1
        assert result.test.passed == 1
        assert result.test.stats.errors == 1
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, SuccessResult)
        assert isinstance(result.test.testsuite.testcases[1].result, ErrorResult)

    def test_syntaxerr1(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql',
                    content=ctx.fixtures.load('mariadb/syntaxerr1/solution.sql'),
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt',
                    content=ctx.fixtures.load('mariadb/syntaxerr1/expected.txt'),
                )
            ],
            include_files=[
                EvaFile(
                    path='import.sql', content=ctx.fixtures.load('mariadb/syntaxerr1/import.sql')
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 2
        assert result.test.passed == 0
        assert result.test.stats.errors == 1
        assert result.test.stats.failures == 1
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, FailureResult)
        assert isinstance(result.test.testsuite.testcases[1].result, ErrorResult)

    def test_syntaxerr2(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql',
                    content=ctx.fixtures.load('mariadb/syntaxerr2/solution.sql'),
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt',
                    content=ctx.fixtures.load('mariadb/syntaxerr2/expected.txt'),
                )
            ],
            include_files=[
                EvaFile(
                    path='import.sql', content=ctx.fixtures.load('mariadb/syntaxerr2/import.sql')
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 2
        assert result.test.passed == 0
        assert result.test.stats.errors == 1
        assert result.test.stats.failures == 1
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, FailureResult)
        assert isinstance(result.test.testsuite.testcases[1].result, ErrorResult)

    def test_timeout1(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql', content=ctx.fixtures.load('mariadb/timeout1/solution.sql')
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt', content=ctx.fixtures.load('mariadb/timeout1/expected.txt')
                )
            ],
            include_files=[
                EvaFile(path='import.sql', content=ctx.fixtures.load('mariadb/timeout1/import.sql'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 1
        assert result.test.testcases == 1
        assert result.test.failed == 1
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 0
        assert isinstance(result.test.testsuite.testcases[0].result, TimeoutResult)

    def test_timeout2(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(
                    path='solution.sql', content=ctx.fixtures.load('mariadb/timeout2/solution.sql')
                )
            ],
            test_files=[
                EvaFile(
                    path='expected.txt', content=ctx.fixtures.load('mariadb/timeout2/expected.txt')
                )
            ],
            include_files=[
                EvaFile(path='import.sql', content=ctx.fixtures.load('mariadb/timeout2/import.sql'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 2
        assert result.test.testcases == 2
        assert result.test.failed == 2
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 1
        assert result.test.stats.pending == 1
        assert isinstance(result.test.testsuite.testcases[0].result, TimeoutResult)
        assert isinstance(result.test.testsuite.testcases[1].result, PendingResult)
