import pytest
from eva.pltform.evaluation.core import EvaFile, EvalPayload
from eva.pltform.format.trex import ErrorResult, FailureResult, SuccessResult
from tests.integration.util.context import TestContext


@pytest.mark.evaluator('python')
class PythonTest:

    def test_passing(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.py', content=ctx.fixtures.load('python/passing/fac.py'))
            ],
            test_files=[
                EvaFile(path='FacTest.py', content=ctx.fixtures.load('python/passing/FacTest.py'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 1.0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 0
        assert result.test.passed == 3
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, SuccessResult)

    def test_syntaxerr(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.py', content=ctx.fixtures.load('python/syntaxerr/fac.py'))
            ],
            test_files=[
                EvaFile(path='FacTest.py', content=ctx.fixtures.load('python/syntaxerr/FacTest.py'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert 'IndentationError: expected an indented block' in result.test.output
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

    def test_failure(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.py', content=ctx.fixtures.load('python/failure/fac.py'))
            ],
            test_files=[
                EvaFile(path='FacTest.py', content=ctx.fixtures.load('python/failure/FacTest.py'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 3
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0
        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, FailureResult)

    def test_infloop(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.py', content=ctx.fixtures.load('python/infloop/fac.py'))
            ],
            test_files=[
                EvaFile(path='FacTest.py', content=ctx.fixtures.load('python/infloop/FacTest.py'))
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 0
        assert result.test.testcases == 0
        assert result.test.failed == 0
        assert result.test.passed == 0
        assert result.test.stats.errors == 0
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

    def test_stackoverflow(self, ctx: TestContext):
        payload = EvalPayload(
            solution_files=[
                EvaFile(path='fac.py', content=ctx.fixtures.load('python/stackoverflow/fac.py'))
            ],
            test_files=[
                EvaFile(
                    path='FacTest.py', content=ctx.fixtures.load('python/stackoverflow/FacTest.py')
                )
            ],
            test=True,
        )
        result = ctx.eval(payload)
        assert result.compilation is None
        assert pytest.approx(result.test.correctness) == 0
        assert len(result.test.testsuite.testcases) == 3
        assert result.test.testcases == 3
        assert result.test.failed == 3
        assert result.test.passed == 0
        assert result.test.stats.errors == 3
        assert result.test.stats.failures == 0
        assert result.test.stats.timeouts == 0
        assert result.test.stats.pending == 0

        for t in result.test.testsuite.testcases:
            assert isinstance(t.result, ErrorResult)
