import os
import typing
from typing import Optional

from pydantic import BaseModel, field_validator


def getenv(var, default=None, cast=None):
    val = os.getenv(var, default)
    return cast(val) if cast is not None and val is not None else val


class GitLabEnvironmentProviderConfig(BaseModel):
    user: Optional[str] = None
    token: Optional[str] = None
    gitlab_url: str
    docker_registry_url: str
    project_id: str


class DockerEnvironmentProviderConfig(BaseModel):
    docker_registry_url: str
    user: Optional[str] = None
    password: Optional[str] = None


class Config(BaseModel):
    provider: str
    img_base_path: str
    cache_ttl: int  # in seconds, -1 for infinite
    provider_conf: Optional[typing.Any] = None

    @staticmethod
    def from_env():
        provider = getenv('EVA_REGISTRY_PROVIDER', 'local')
        img_base_path = getenv('EVA_REGISTRY_IMG_BASE_PATH', 'eva/environments')

        provider_conf = None
        if provider == 'gitlab':
            provider_conf = GitLabEnvironmentProviderConfig(
                user=getenv('EVA_REGISTRY_GITLAB_USER'),
                token=getenv('EVA_REGISTRY_GITLAB_TOKEN'),
                gitlab_url=getenv('EVA_REGISTRY_GITLAB_URL'),
                docker_registry_url=getenv('EVA_REGISTRY_GITLAB_DOCKER_REGISTRY_URL'),
                project_id=getenv('EVA_REGISTRY_GITLAB_PROJECT_ID'),
            )

        if provider == 'docker':
            provider_conf = DockerEnvironmentProviderConfig(
                user=getenv('EVA_REGISTRY_DOCKER_USER'),
                password=getenv('EVA_REGISTRY_DOCKER_PASSWORD'),
                docker_registry_url=getenv('EVA_REGISTRY_DOCKER_DOCKER_REGISTRY_URL'),
            )

        return Config(
            provider=provider,
            img_base_path=img_base_path,
            cache_ttl=getenv('EVA_REGISTRY_CACHE_TTL', '300', cast=int),
            provider_conf=provider_conf,
        )

    @field_validator('provider')
    @classmethod
    def check_provider(cls, provider: str) -> str:
        assert provider in ('gitlab', 'docker', 'local'), f'unknown provider: {provider}'
        return provider
