import datetime
import logging
from typing import List

import requests
from eva.registry.environ.cache import EnvironmentsCache
from eva.registry.environ.model import Environment
from eva.registry.environ.provider import EnvironmentProvider


class DockerEnvironmentProvider(EnvironmentProvider):
    TIMEOUT = 20

    def __init__(
        self, cache: EnvironmentsCache, img_base_path: str, docker_registry_url: str, creds=None
    ):
        self.cache = cache
        self.img_base_path = img_base_path
        self.docker_registry_url = docker_registry_url
        self.creds = creds

        self.logger = logging.getLogger(__name__)

    def get_tags(self, repository_path):
        api_url = f'{self.docker_registry_url}/v2/{repository_path}/tags/list'
        response = requests.get(
            api_url,
            headers={
                'Accept': 'application/vnd.docker.distribution.manifest.v2+json',
            },
            auth=self.creds,
            timeout=self.TIMEOUT,
        )
        response.raise_for_status()
        data = response.json()
        return data['tags']

    def get_image_digest(self, repository_path, tag):
        api_url = f'{self.docker_registry_url}/v2/{repository_path}/manifests/{tag}'
        response = requests.get(
            api_url,
            headers={
                'Accept': 'application/vnd.docker.distribution.manifest.v2+json',
            },
            auth=self.creds,
            timeout=self.TIMEOUT,
        )
        response.raise_for_status()
        data = response.json()
        digest = data['config']['digest']
        return digest

    def get_image_metadata(self, repository_path, digest):
        api_url = f'{self.docker_registry_url}/v2/{repository_path}/blobs/{digest}'
        response = requests.get(
            api_url,
            headers={
                'Accept': 'application/vnd.docker.distribution.manifest.v2+json',
            },
            auth=self.creds,
            timeout=self.TIMEOUT,
        )
        response.raise_for_status()
        data = response.json()
        return data

    def get_evaluator_repositories(self):
        headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}

        # todo: implement paging
        url = f'{self.docker_registry_url}/v2/_catalog' f'?per_page=200'
        response = requests.get(url, headers=headers, auth=self.creds, timeout=self.TIMEOUT)

        response.raise_for_status()
        data = response.json()
        return [d for d in data['repositories'] if self.img_base_path in d]

    def get_envs(self) -> (List[Environment], datetime.datetime):
        if not self.cache.expired():
            return self.cache.get(), self.cache.last_update

        environments = []
        repository_paths = self.get_evaluator_repositories()

        for repository_path in repository_paths:
            image_name = repository_path.split('/')[-1]
            tags = self.get_tags(repository_path)

            for tag in tags:
                if tag == 'latest':
                    digest = self.get_image_digest(repository_path, tag)
                    image_metadata = self.get_image_metadata(repository_path, digest)
                    labels = image_metadata['config']['Labels'] or {}

                    description = labels.get('DESCRIPTION', 'undefined')
                    maintainer = labels.get('MAINTAINER', 'undefined')

                    environments.append(
                        Environment(
                            id=image_name,
                            description=description,
                            maintainer=maintainer,
                            tag=tag,
                        )
                    )

        self.cache.set(environments)
        return environments, self.cache.last_update
