import base64
import datetime
import json
import logging
from typing import List

import requests
from eva.registry.environ.cache import EnvironmentsCache
from eva.registry.environ.model import Environment
from eva.registry.environ.provider import EnvironmentProvider


class GitLabEnvironmentProvider(EnvironmentProvider):
    TIMEOUT = 20

    def __init__(
        self,
        cache: EnvironmentsCache,
        img_base_path: str,
        gitlab_url: str,
        docker_registry_url: str,
        project_id: str,
        gitlab_creds=None,
    ):
        self.cache = cache
        self.img_base_path = img_base_path
        self.gitlab_url = gitlab_url
        self.docker_registry_url = docker_registry_url
        self.project_id = project_id
        self.gitlab_creds = gitlab_creds

        self.token_map = {}
        self.logger = logging.getLogger(__name__)

    def get_evaluator_repositories(self):
        headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}

        if self.gitlab_creds:
            headers['PRIVATE-TOKEN'] = self.gitlab_creds[1]

        # todo: implement paging
        url = (
            f'{self.gitlab_url}/api/v4/projects/{self.project_id}/registry/repositories'
            f'?per_page=200'
        )
        response = requests.get(url, headers=headers, timeout=self.TIMEOUT)

        response.raise_for_status()
        data = response.json()
        return [d['path'] for d in data if self.img_base_path in d['location']]

    def get_tags(self, repository_path, token):
        api_url = f'{self.docker_registry_url}/v2/{repository_path}/tags/list'
        response = requests.get(
            api_url,
            headers={
                'Accept': 'application/vnd.docker.distribution.manifest.v2+json',
                'Authorization': f'Bearer {token}',
            },
            timeout=self.TIMEOUT,
        )
        response.raise_for_status()
        data = response.json()
        return data['tags']

    def get_token(self, repository_path):
        if repository_path in self.token_map and self.token_valid(self.token_map[repository_path]):
            self.logger.debug(f'token exists and valid for {repository_path}, not renewing')
            return self.token_map[repository_path]

        api_url = (
            f'{self.gitlab_url}/jwt/auth?client_id=docker&offline_token=true'
            f'&service=container_registry&scope=repository:{repository_path}:push,pull'
        )
        response = requests.get(api_url, auth=self.gitlab_creds, timeout=self.TIMEOUT)
        response.raise_for_status()
        data = response.json()
        token = data['token']
        self.logger.debug(f'token updated for {repository_path}')

        self.token_map[repository_path] = token
        return token

    def token_valid(self, token):
        _, payload, _ = token.split('.')
        decoded = base64.b64decode(payload + '==').decode('utf-8')
        payload_dict = json.loads(decoded)
        current_time = datetime.datetime.now()
        current_timestamp = int(current_time.timestamp())
        return current_timestamp < payload_dict['exp']

    def get_image_digest(self, repository_path, token, tag):
        api_url = f'{self.docker_registry_url}/v2/{repository_path}/manifests/{tag}'
        response = requests.get(
            api_url,
            headers={
                'Accept': 'application/vnd.docker.distribution.manifest.v2+json',
                'Authorization': f'Bearer {token}',
            },
            timeout=self.TIMEOUT,
        )
        response.raise_for_status()
        data = response.json()
        digest = data['config']['digest']
        return digest

    def get_image_metadata(self, repository_path, digest, token):
        api_url = f'{self.docker_registry_url}/v2/{repository_path}/blobs/{digest}'
        response = requests.get(
            api_url,
            headers={
                'Accept': 'application/vnd.docker.distribution.manifest.v2+json',
                'Authorization': f'Bearer {token}',
            },
            timeout=self.TIMEOUT,
        )
        response.raise_for_status()
        data = response.json()
        return data

    def get_envs(self) -> (List[Environment], datetime.datetime):
        if not self.cache.expired():
            return self.cache.get(), self.cache.last_update

        repository_paths = self.get_evaluator_repositories()

        environments = []
        for repository_path in repository_paths:
            image_name = repository_path.split('/')[-1]
            token = self.get_token(repository_path)
            tags = self.get_tags(repository_path, token)

            for tag in tags:
                if tag == 'latest':
                    digest = self.get_image_digest(repository_path, token, tag)
                    image_metadata = self.get_image_metadata(repository_path, digest, token)
                    labels = image_metadata['config']['Labels'] or {}

                    description = labels.get('DESCRIPTION', 'undefined')
                    maintainer = labels.get('MAINTAINER', 'undefined')

                    environments.append(
                        Environment(
                            id=image_name,
                            description=description,
                            maintainer=maintainer,
                            tag=tag,
                        )
                    )

        self.cache.set(environments)
        return environments, self.cache.last_update
