import logging
import math

from eva.registry.config import Config
from eva.registry.dto import GetStackResponse, StacksResponse
from eva.registry.environ import (
    EnvironmentsCache,
    GitLabEnvironmentProvider,
    LocalDockerEnvironmentProvider,
)
from eva.registry.environ.docker import DockerEnvironmentProvider
from eva.registry.logsetup import setup_logging
from eva.registry.stack import (
    EvaluationStackNotFoundError,
    EvaluationStackProvider,
    EvaluationStackQuery,
)
from fastapi import FastAPI, HTTPException, Request, status
from starlette.responses import JSONResponse

setup_logging()
logger = logging.getLogger(__name__)

config = Config.from_env()
logger.info(
    f"config: {config.model_dump_json(indent=4, exclude={'provider_conf': ['token', 'password'] })}"
)

cache = EnvironmentsCache(ttl=config.cache_ttl if config.cache_ttl != -1 else math.inf)
if config.provider == 'gitlab':
    GITLAB = config.provider_conf
    envs = GitLabEnvironmentProvider(
        cache=cache,
        img_base_path=config.img_base_path,
        gitlab_url=GITLAB.gitlab_url,
        docker_registry_url=GITLAB.docker_registry_url,
        project_id=GITLAB.project_id,
        gitlab_creds=((GITLAB.user, GITLAB.token) if GITLAB.user and GITLAB.token else None),
    )

if config.provider == 'docker':
    DOCKER = config.provider_conf
    envs = DockerEnvironmentProvider(
        cache=cache,
        img_base_path=config.img_base_path,
        docker_registry_url=DOCKER.docker_registry_url,
        creds=((DOCKER.user, DOCKER.password) if DOCKER.user and DOCKER.password else None),
    )

if config.provider == 'local':
    envs = LocalDockerEnvironmentProvider(cache=cache, img_base_path=config.img_base_path)

stack_provider = EvaluationStackProvider(envs)

app = FastAPI()


@app.exception_handler(Exception)
async def custom_exception_handler(request: Request, exc: Exception):
    # pylint: disable=unused-argument
    logger.exception('Uncaught exception occurred!')
    return JSONResponse(
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
        content={'detail': 'Something went wrong.'},
    )


@app.get('/stacks')
async def root() -> StacksResponse:
    stacks, last_update = stack_provider.get_stacks()
    return StacksResponse(stacks=stacks, last_update=last_update)


@app.get('/stacks/{ref}')
async def get_evaluator(ref: str) -> GetStackResponse:
    try:
        stack, last_update = EvaluationStackQuery(ref).find(stack_provider)
        return GetStackResponse(stack=stack, last_update=last_update)
    except EvaluationStackNotFoundError as e:
        raise HTTPException(status_code=404, detail=str(e)) from e
