package de.hsrm.sls.cexf.jacoco;

import de.hsrm.sls.ejunit.common.DOMElementWriter;
import de.hsrm.sls.ejunit.common.TestcaseIdentifier;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.jacoco.core.JaCoCo;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class CEXFSerializer implements JacocoSerializer {

    private static final String E_COVERAGE = "coverage";
    private static final String E_TESTSUITE = "testsuite";
    private static final String E_LINE = "line";
    private static final String E_TESTCASE = "testcase";
    private static final String E_FILE = "file";
    private static final String A_CLASSNAME = "classname";
    private static final String A_NAME = "name";
    private static final String A_FQCN = "fqcn";
    private static final String A_TOOL = "tool";
    private static final String A_VERSION = "version";
    private static final String A_NR = "nr";
    private static final String A_COVERED = "covered";
    private static final String A_MISSED = "missed";
    private static final String A_TYPE = "type";

    private final String filePath;
    private final JacocoCoverageParser coverageParser;
    private final CEXFLineType lineTypeFilter;

    public CEXFSerializer(String filePath, JacocoCoverageParser coverageParser, CEXFLineType lineTypeFilter) {
        this.filePath = filePath;
        this.coverageParser = coverageParser;
        this.lineTypeFilter = lineTypeFilter;
    }

    public void serialize(JacocoState state) {
        var doc = getDocumentBuilder().newDocument();
        var rootElement = doc.createElement(E_COVERAGE);
        rootElement.setAttribute(A_TOOL, "jacoco");
        rootElement.setAttribute(A_VERSION, JaCoCo.VERSION);
        doc.appendChild(rootElement);

        var testsuiteElement = doc.createElement(E_TESTSUITE);
        rootElement.appendChild(testsuiteElement);

        for (var e : state.getExecutionDataMap().entrySet()) {
            var execData = e.getValue();
            var testcaseIdentifier = TestcaseIdentifier.fromString(e.getKey());

            if (execData == null) {
                continue;
            }

            var coverageData = coverageParser.getCoverage(execData);
            for (var pack : coverageData.getPackages()) {
                for (var f : pack.getSourceFiles()) {
                    if (!f.getName().endsWith("Test")) {
                        var testElement = doc.createElement(E_TESTCASE);
                        testElement.setAttribute(A_CLASSNAME, testcaseIdentifier.className());
                        testElement.setAttribute(A_NAME, testcaseIdentifier.methodName());
                        testsuiteElement.appendChild(testElement);

                        var fullFileName = f.getPackageName().isEmpty() ? f.getName()
                            : "%s/%s".formatted(f.getPackageName(), f.getName());
                        var fqcn = fullFileName.replace(".java", "").replace("/", ".");
                        var fileElement = doc.createElement(E_FILE);
                        fileElement.setAttribute(A_NAME, fullFileName);
                        fileElement.setAttribute(A_FQCN, fqcn);
                        testElement.appendChild(fileElement);

                        for (var i = f.getFirstLine(); i < f.getLastLine(); i++) {

                            if(lineTypeFilter == null || lineTypeFilter == CEXFLineType.INSTRUCTION) {
                                fileElement.appendChild(createLineElement(
                                    doc,
                                    i,
                                    CEXFLineType.INSTRUCTION,
                                    f.getLine(i).getInstructionCounter().getCoveredCount(),
                                    f.getLine(i).getInstructionCounter().getMissedCount()
                                ));
                            }

                            if(lineTypeFilter == null || lineTypeFilter == CEXFLineType.BRANCH) {
                                fileElement.appendChild(createLineElement(
                                    doc,
                                    i,
                                    CEXFLineType.BRANCH,
                                    f.getLine(i).getBranchCounter().getCoveredCount(),
                                    f.getLine(i).getBranchCounter().getMissedCount()
                                ));
                            }

                            if(lineTypeFilter == null || lineTypeFilter == CEXFLineType.LINE) {
                                fileElement.appendChild(createLineElement(
                                    doc,
                                    i,
                                    CEXFLineType.LINE,
                                    f.getLine(i).getInstructionCounter().getCoveredCount(),
                                    f.getLine(i).getInstructionCounter().getMissedCount()
                                ));
                            }
                        }
                    }
                }
            }
        }

        new DOMElementWriter().write(doc, filePath);
    }

    private Element createLineElement(Document doc, int lineNumber, CEXFLineType type, int covered,
        int missed) {
        var lineElement = doc.createElement(E_LINE);
        lineElement.setAttribute(A_NR, String.valueOf(lineNumber));
        lineElement.setAttribute(A_TYPE, type.getIdentifier());
        lineElement.setAttribute(A_COVERED, String.valueOf(covered));
        lineElement.setAttribute(A_MISSED, String.valueOf(missed));
        return lineElement;
    }

    private static DocumentBuilder getDocumentBuilder() {
        try {
            return DocumentBuilderFactory.newInstance().newDocumentBuilder();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
