package de.hsrm.sls.cexf.jacoco;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import org.jacoco.core.analysis.Analyzer;
import org.jacoco.core.analysis.CoverageBuilder;
import org.jacoco.core.analysis.IBundleCoverage;
import org.jacoco.core.data.ExecutionDataStore;
import org.jacoco.core.tools.ExecFileLoader;

public class JacocoCoverageParser {

    /**
     * List of class files that should be analyzed. Class files not added to this list will not
     * appear in the coverage data.
     */
    private List<String> filterFiles = new ArrayList<>();

    /**
     * @param filterFiles .class files of the classes that should be included in parsed coverage
     *                    date.
     */
    public JacocoCoverageParser(List<String> filterFiles) {
        this.filterFiles = filterFiles;
    }

    /**
     * gets the coverage data for the class identified by the fqcn.
     *
     * @param executionData raw jacoco data (stored in *.exec files)
     * @return
     */
    public IBundleCoverage getCoverage(byte[] executionData) {
        try {
            final ExecFileLoader loader = new ExecFileLoader();
            loader.load(new ByteArrayInputStream(executionData));
            return analyze(loader.getExecutionDataStore());
        } catch (IOException e) {
            throw new RuntimeException("Failed to get coverage data", e);
        }
    }


    private IBundleCoverage analyze(ExecutionDataStore data) throws IOException {
        final CoverageBuilder builder = new CoverageBuilder();
        final Analyzer analyzer = new Analyzer(data, builder);

        for (var f : filterFiles) {
            var inputStream = getClassAsStream(f);
            var pathToClass = getPathToClass(f);

            if(inputStream == null || pathToClass == null) {
                System.out.println("WARNING: class not found: %s".formatted(f));
                continue;
            }

            analyzer.analyzeClass(inputStream, pathToClass);
        }

        if (!builder.getNoMatchClasses().isEmpty()) {
            throw new RuntimeException("Some classes do not match with execution data");

      /*
      for (final IClassCoverage c : nomatch) {
        out.printf(
            "[WARN] Execution data for class %s does not match.%n",
            c.getName());
      }*/
        }

        return builder.getBundle("Report");
    }

    /**
     * Gets the content of a class as input stream.
     *
     * @param className FQCN of the class. The class has to be on the class path.
     * @return
     */
    private InputStream getClassAsStream(String className) {
        var inputStream = getClass().getClassLoader()
            .getResourceAsStream(className.replace('.', '/') + ".class");
        if (inputStream == null) {
            return null;
        }
        return inputStream;
    }

    /**
     * Gets the absolute path of a class file.
     *
     * @param className FQCN of the class. The class has to be on the class path.
     * @return path to the class file or null if
     * a) the class could not be found or
     * b) if static initializers in the class have thrown an exception
     */
    private String getPathToClass(String className) {
        Class<?> clazz;
        try {
            clazz = Class.forName(className);
        } catch (ClassNotFoundException e) {
            return null;
        } catch(Error e) {
            // static initializers in the class may throw an exception, e.g. static String NEW_LINE = System.getProperty("");
            // in these cases we cannot load the class, so we will treat these cases as "not found".
            // todo: do we really need to load the class to get the path?
            return null;
        }

        URL classURL = clazz.getResource(clazz.getSimpleName() + ".class");
        if (classURL == null) {
            throw new RuntimeException("path to class %s not found".formatted(className));
        }

        return URLDecoder.decode(classURL.getFile(), StandardCharsets.UTF_8);
    }
}
