package de.hsrm.sls.cexf.jacoco;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class RunArgs {

    private String outPath;
    private List<String> filter;
    private String directoryPath;
    private CEXFLineType lineTypeFilter;

    public String getOutPath() {
        return outPath;
    }

    public void setOutPath(String outPath) {
        this.outPath = outPath;
    }

    public List<String> getFilter() {
        return filter;
    }

    public void setFilter(List<String> filter) {
        this.filter = filter;
    }

    public String getDirectoryPath() {
        return directoryPath;
    }

    public void setDirectoryPath(String directoryPath) {
        this.directoryPath = directoryPath;
    }

    public CEXFLineType getLineTypeFilter() {
        return lineTypeFilter;
    }

    public void setLineTypeFilter(CEXFLineType lineTypeFilter) {
        this.lineTypeFilter = lineTypeFilter;
    }

    private static final String OPT_CEXF_OUT_PATH = "out";
    private static final String OPT_CEXF_FILTER = "file";
    private static final String OPT_CEXF_LINE_TYPE = "ltype";

    private static CommandLine parseArgs(String[] args) {
        Options options = new Options();

        var cexfOutPath = new Option("o", OPT_CEXF_OUT_PATH, true,
            "Output path for the resulting CEXF report.");
        cexfOutPath.setRequired(false);
        options.addOption(cexfOutPath);

        var cexfFilter = new Option("ff", OPT_CEXF_FILTER, true,
            "Comma separated list of FQCNs for filtering code coverage. The classes need to be on the class path. If no classes with the specified FQCNs are found on the class path, the report will be empty.");
        cexfFilter.setRequired(true);
        options.addOption(cexfFilter);

        var lineTypes = Arrays.stream(CEXFLineType.values()).map(CEXFLineType::getIdentifier).collect(
            Collectors.joining(", "));
        var cexfLineTypeFilter = new Option("lt", OPT_CEXF_LINE_TYPE, true,
            "Code coverage granularity to include in the report. One of the following: %s. If not specified, all types will be included in the report.".formatted(lineTypes));
        cexfLineTypeFilter.setRequired(false);
        options.addOption(cexfLineTypeFilter);

        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        CommandLine cmd = null;

        try {
            cmd = parser.parse(options, args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp(
                "ejunit-jacoco [OPTIONS] directory\nParses all *.exec files generated by JaCoCo in the given directory and generates a CEXF report.\nOPTIONS:",
                options);
            System.exit(1);
        }

        return cmd;
    }

    public static RunArgs parse(String[] args) {
        var cmd = parseArgs(args);

        var cexfOutPath = cmd.getOptionValue(OPT_CEXF_OUT_PATH, "cexf_report.xml");
        var cexfFilter = cmd.getOptionValue(OPT_CEXF_FILTER);
        var filter = cexfFilter != null && !cexfFilter.isEmpty()
            ? Arrays.stream(cexfFilter.split(",")).toList()
            : new ArrayList<String>();

        var cexfLineType = cmd.getOptionValue(OPT_CEXF_LINE_TYPE, null);
        var lineType = Arrays.stream(CEXFLineType.values()).filter(lt -> lt.getIdentifier().equals(cexfLineType)).findFirst().orElse(null);

        var directory = cmd.getArgs().length > 0 ? cmd.getArgs()[0] : null;

        if (directory == null) {
            throw new IllegalArgumentException("directory is required.");
        }

        var params = new RunArgs();
        params.setOutPath(cexfOutPath);
        params.setFilter(filter);
        params.setDirectoryPath(directory);
        params.setLineTypeFilter(lineType);
        return params;
    }
}
