package de.hsrm.sls.cexf.openclover;

import static clover.com.google.common.collect.Maps.newHashMap;
import static com.atlassian.clover.reporters.CommandLineArgProcessors.InitString;
import static com.atlassian.clover.reporters.CommandLineArgProcessors.OutputFile;

import clover.com.google.common.collect.Iterables;
import clover.com.google.common.collect.Lists;
import com.atlassian.clover.Logger;
import com.atlassian.clover.api.CloverException;
import com.atlassian.clover.api.command.ArgProcessor;
import com.atlassian.clover.api.registry.FileInfo;
import com.atlassian.clover.api.registry.PackageInfo;
import com.atlassian.clover.model.XmlNames;
import com.atlassian.clover.registry.entities.FullFileInfo;
import com.atlassian.clover.registry.entities.FullPackageInfo;
import com.atlassian.clover.registry.entities.FullProjectInfo;
import com.atlassian.clover.registry.entities.LineInfo;
import com.atlassian.clover.registry.entities.TestCaseInfo;
import com.atlassian.clover.reporters.CloverReportConfig;
import com.atlassian.clover.reporters.CloverReporter;
import com.atlassian.clover.reporters.Current;
import com.atlassian.clover.reporters.Format;
import com.atlassian.clover.reporters.xml.XMLWriter;
import com_atlassian_clover.CloverVersionInfo;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

/**
 * Based on {@link com.atlassian.clover.reporters.xml.XMLReporter}. Improvements:
 * <ul>
 *   <li>Coverage per testcase</li>
 *   <li>CEXF format</li>
 * </ul>
 *  OpenClover does not print per-test coverage information in the XML report.
 *  See <a href="https://stackoverflow.com/questions/61268633/openclover-parsing-the-xml-output-java">Stackoverflow</a>
 *  and <a href="https://community.atlassian.com/t5/Clover-questions/How-to-get-clover-coverage-percentage-for-one-test-case-on-a/qaq-p/1348720#M2200">Jira Issue</a>
 */
public class CEXFReport extends CloverReporter {

    public static final String E_TESTSUITE = "testsuite";
    public static final String E_TESTCASE = "testcase";
    public static final String A_CLASS_NAME = "classname";
    public static final String A_COVERED = "covered";
    public static final String A_MISSED = "missed";
    public static final String A_FQCN = "fqcn";


    @SuppressWarnings("unchecked")
    private static final List<ArgProcessor<? extends Current>> mandatoryArgProcessors = Lists.newArrayList(
        InitString,
        ArgProcessors.FileFilter
    );

    @SuppressWarnings("unchecked")
    private static final List<ArgProcessor<? extends Current>> optionalArgProcessors = Lists.newArrayList(
        OutputFile,
        ArgProcessors.LineTypeFilter
    );

    private static final List<ArgProcessor<? extends Current>> allArgProcessors = Lists.newArrayList(
        Iterables.concat(mandatoryArgProcessors, optionalArgProcessors));


    public CEXFConfig config() {
        return (CEXFConfig) reportConfig;
    }

    public CEXFReport(CloverReportConfig config) throws CloverException {
        super(config.getCoverageDatabase(), config);
    }

    public int executeImpl() throws CloverException {
        try {
            XMLWriter out = initWriter();

            Logger.getInstance().info("Writing report to '" + reportConfig.getOutFile() + "'");

            out.writeXMLDecl();

            Map<String, String> attribs = newHashMap();
            attribs.put("tool", "clover");
            attribs.put("version", CloverVersionInfo.RELEASE_NUM);

            out.writeElementStart(XmlNames.E_COVERAGE, attribs);
            out.writeElementStart(E_TESTSUITE, newHashMap());
            writeCoverage(out, database.getAppOnlyModel());
            out.writeElementEnd(E_TESTSUITE);
            out.writeElementEnd(XmlNames.E_COVERAGE);

            out.close();

            return 0;
        } catch (IOException e) {
            throw new CloverException("IO Exception: " + e.getMessage());
        }
    }

    private XMLWriter initWriter() throws IOException {
        OutputStream os;

        final File outFile = reportConfig.getOutFile();
        if (outFile.getParent() != null && !outFile.getParentFile().exists()) {
            outFile.getParentFile().mkdirs();
        }

        os = new BufferedOutputStream(Files.newOutputStream(outFile.toPath()));
        return new XMLWriter(os, "UTF-8");
    }

    private void writeCoverage(XMLWriter out, FullProjectInfo proj)
        throws IOException {
        Map<String, String> attribs = newHashMap();
        if (reportConfig.getTitle() != null) {
            attribs.put(XmlNames.A_NAME, reportConfig.getTitle());
        }

        long ts = database.getRecordingTimestamp();
        attribs.put(XmlNames.A_TIMESTAMP, String.valueOf(ts));

        var tests = database.getCoverageData().getTests().stream()
            .sorted(Comparator.comparing(TestCaseInfo::getStartTime)).toList();
        for (var test : tests) {
            attribs = newHashMap();
            attribs.put(A_CLASS_NAME, test.getRuntimeTypeName());
            attribs.put(XmlNames.A_NAME, test.getTestName());
            out.writeElementStart(E_TESTCASE, attribs);

            List<? extends PackageInfo> packages = proj.getAllPackages();
            for (PackageInfo packageInfo : packages) {
                FullPackageInfo pkg = (FullPackageInfo) packageInfo;
                writeFilesForPkg(out, pkg, test);
            }
            out.writeElementEnd(E_TESTCASE);
        }
    }

    private void writeFilesForPkg(XMLWriter out, FullPackageInfo pkg, TestCaseInfo test)
        throws IOException {
        final List<? extends FileInfo> files = pkg.getFiles();

        for (FileInfo fileInfo : files) {

            final FullFileInfo file = (FullFileInfo) fileInfo;

            var fqcn = file.getPackagePath().replace(".java", "").replace("/", ".");
            if (!config().getFiles().contains(fqcn)) {
                continue;
            }

            final Map<String, String> attribs = newHashMap();

            attribs.put(XmlNames.A_NAME, file.getPackagePath());
            attribs.put(A_FQCN, fqcn);
            out.writeElementStart(XmlNames.E_FILE, attribs);
            writePerTestLineInfo(out, file, test);
            out.writeElementEnd(XmlNames.E_FILE);
        }
    }

    private void writePerTestLineInfo(XMLWriter out, FullFileInfo finfo, TestCaseInfo test)
        throws IOException {
        final int linecount = finfo.getLineCount();
        final LineInfo[] linfo = finfo.getLineInfo(config().isShowLambdaFunctions(), config().isShowInnerFunctions());

        var coveragePerTestMap = database.getCoverageData().getPerTestCoverage()
            .mapTestsAndCoverageForFile(finfo);
        var testCoverage = coveragePerTestMap.get(test);
        if (testCoverage == null) {
            return;
        }

        for (int i = 1; i <= linecount; i++) {
            LineInfo info = linfo[i];

            if (linfo[i] != null) {
                var classes = Arrays.stream(info.getClassStarts()).toList();

                var stmts = Arrays.stream(info.getStatements()).toList();
                if (stmts.size() > 0) {
                    var hitStmts = stmts.stream().filter(s -> testCoverage.get(s.getDataIndex()));
                    var missedStmts = stmts.stream()
                        .filter(s -> !testCoverage.get(s.getDataIndex()));

                    final Map<String, String> attribs = newHashMap();
                    attribs.put("nr", String.valueOf(i));
                    attribs.put(A_COVERED, String.valueOf(hitStmts.count()));
                    attribs.put(A_MISSED, String.valueOf(missedStmts.count()));

                    if(config().getLineType() == null || config().getLineType() == CEXFLineType.STATEMENT) {
                        attribs.put(XmlNames.A_LINETYPE, CEXFLineType.STATEMENT.getIdentifier());
                        out.writeElement(XmlNames.E_LINE, attribs);
                    }

                    if(config().getLineType() == null || config().getLineType() == CEXFLineType.LINE) {
                        attribs.put(XmlNames.A_LINETYPE, CEXFLineType.LINE.getIdentifier());
                        out.writeElement(XmlNames.E_LINE, attribs);
                    }
                }

                var methods = Arrays.stream(info.getMethodStarts()).toList();
                if (methods.size() > 0) {
                    if(config().getLineType() == null || config().getLineType() == CEXFLineType.METHOD) {
                        var hitMethods = methods.stream()
                            .filter(s -> testCoverage.get(s.getDataIndex()));
                        var missedMethods = methods.stream()
                            .filter(s -> !testCoverage.get(s.getDataIndex()));

                        final Map<String, String> attribs = newHashMap();
                        attribs.put("nr", String.valueOf(i));
                        attribs.put(XmlNames.A_LINETYPE, CEXFLineType.METHOD.getIdentifier());
                        attribs.put(A_COVERED, String.valueOf(hitMethods.count()));
                        attribs.put(A_MISSED, String.valueOf(missedMethods.count()));
                        out.writeElement(XmlNames.E_LINE, attribs);
                    }
                }

                if (classes.size() > 0) {
                    if(config().getLineType() == null || config().getLineType() == CEXFLineType.CLASS) {
                        var hitClasses = classes.stream()
                            .filter(s -> testCoverage.get(s.getDataIndex()));
                        var missedClasses = classes.stream()
                            .filter(s -> !testCoverage.get(s.getDataIndex()));

                        final Map<String, String> attribs = newHashMap();
                        attribs.put("nr", String.valueOf(i));
                        attribs.put(XmlNames.A_LINETYPE, CEXFLineType.CLASS.getIdentifier());
                        attribs.put(A_COVERED, String.valueOf(hitClasses.count()));
                        attribs.put(A_MISSED, String.valueOf(missedClasses.count()));
                        out.writeElement(XmlNames.E_LINE, attribs);
                    }
                }

                var branches = Arrays.stream(info.getBranches()).toList();
                if (branches.size() > 0) {
                    if(config().getLineType() == null || config().getLineType() == CEXFLineType.BRANCH) {
                        var hitBranches = branches.stream()
                            .filter(s -> testCoverage.get(s.getDataIndex()));
                        var missedBranches = branches.stream()
                            .filter(s -> !testCoverage.get(s.getDataIndex()));

                        final Map<String, String> attribs = newHashMap();
                        attribs.put("nr", String.valueOf(i));
                        attribs.put(XmlNames.A_LINETYPE, CEXFLineType.BRANCH.getIdentifier());
                        attribs.put(A_COVERED, String.valueOf(hitBranches.count()));
                        attribs.put(A_MISSED, String.valueOf(missedBranches.count()));
                        out.writeElement(XmlNames.E_LINE, attribs);
                    }
                }
            }
        }
    }

    private static CEXFConfig processArgs(String[] args) {
        var cfg = new CEXFConfig();
        cfg.setFormat(Format.DEFAULT_XML);
        cfg.setIncludeFailedTestCoverage(true);
        cfg.setOutFile(new File("cexf_report.xml"));

        try {
            int i = 0;

            while (i < args.length) {
                for (ArgProcessor argProcessor : allArgProcessors) {
                    if (argProcessor.matches(args, i)) {
                        i = argProcessor.process(args, i, cfg);
                    }
                }
                i++;
            }

            if (!cfg.validate()) {
                usage(cfg.getValidationFailureReason());
                cfg = null;
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            usage("Missing a parameter.");
            cfg = null;
        }
        return cfg;
    }


    private static void usage(String msg) {
        System.err.println();
        if (msg != null) {
            System.err.println("  *** ERROR: " + msg);
        }
        System.err.println();

        System.err.println(HelpBuilder.buildHelp(CEXFReport.class, mandatoryArgProcessors,
            optionalArgProcessors));

        System.err.println();
    }

    public static void main(String[] args) {
        loadLicense();
        System.exit(runReport(args));
    }

    public static int runReport(String[] args) {
        CloverReportConfig config = processArgs(args);


        if (canProceedWithReporting(config)) {
            try {
                return new CEXFReport(config).execute();
            } catch (CloverException e) {
                Logger.getInstance()
                    .error("An error occurred while generating the report: " + e.getMessage(), e);
            }
        }
        return 1;
    }
}