package de.hsrm.sls.ejunit.common;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.function.Consumer;
import java.util.function.Supplier;
import java.util.stream.Collectors;

/**
 * Responsible for capturing stdout/stderr streams into strings.
 */
public class StdCapture {

    private PrintStream originalOut;
    private SlidingStream slidingStream;
    private PrintStream outputStream;
    private PrintStream captureStream;

    private Supplier<PrintStream> streamSupplier;
    private Consumer<PrintStream> restorer;

    public StdCapture(Supplier<PrintStream> streamSupplier, Consumer<PrintStream> restorer,
        int maxSize) {
        slidingStream = new SlidingStream(new SlidingBuffer<>(maxSize));
        outputStream = new PrintStream(slidingStream);
        captureStream = new PrintStream(outputStream);
        this.streamSupplier = streamSupplier;
        this.restorer = restorer;
    }

    public void startCapture() {
        originalOut = streamSupplier.get();
        restorer.accept(new TeePrintStream(originalOut, captureStream));
    }

    public void stopCapture() {
        restorer.accept(originalOut);
        captureStream.flush();
    }

    @Override
    public String toString() {
        return slidingStream.toString();
    }

    private static class TeePrintStream extends PrintStream {

        private final PrintStream secondStream;

        public TeePrintStream(PrintStream mainStream, PrintStream secondStream) {
            super(mainStream);
            this.secondStream = secondStream;
        }

        @Override
        public void write(byte[] buf, int off, int len) {
            super.write(buf, off, len);
            secondStream.write(buf, off, len);
        }

        @Override
        public void flush() {
            super.flush();
            secondStream.flush();
        }

        @Override
        public void close() {
            super.close();
            secondStream.close();
        }
    }


    /**
     * Testcases may call a method in the code under test that starts a long running thread and terminate.
     * This may lead to interference in the captured outputs If this long running thread
     * prints to stdout/stderr and stays alive while other testcases are executed.
     * process and toString() are synchronized because this class needs to be thread safe.
     */
    private static class SlidingStream extends ByteArrayOutputStream {

        private final SlidingBuffer<Character> slidingBuffer;

        public SlidingStream(SlidingBuffer<Character> slidingBuffer) {
            super();
            this.slidingBuffer = slidingBuffer;
        }

        @Override
        public void write(byte[] buf, int off, int len) {
            super.write(buf, off, len);
            process();
        }

        @Override
        public void flush() throws IOException {
            super.flush();
            process();
        }

        private synchronized void process() {
            var lines = new String(this.buf, 0, this.count);
            for (var l : lines.toCharArray()) {
                slidingBuffer.add(l);
            }
            this.reset();
        }

        @Override
        public synchronized String toString() {
            var s = slidingBuffer.getBuffer().stream().map(Object::toString)
                .collect(Collectors.joining());
            return slidingBuffer.getTruncated() != 0 ? "[%s chars truncated] ... %s".formatted(
                slidingBuffer.getTruncated(), s) : s;
        }
    }
}