package de.hsrm.sls.ejunit.common;

/**
 * The Java Language Specification does not guarantee that a status code != 0 is returned in the
 * case of uncaught exceptions. This handler should be configured at the entry point of a program
 * to guarantee exiting with status 1 if runtime exceptions occur:
 *
 * {@code Thread.currentThread().setUncaughtExceptionHandler(new UncaughtExceptionHandler()); }
 *
 * Do not use {@code Thread.setDefaultUncaughtExceptionHandler} because exceptions occuring in threads that are started
 * from code under test are no errors of the runner and would otherwise lead to status code 1.
 *
 * @see <a href="https://softwareengineering.stackexchange.com/questions/20573/why-does-java-exit-with-success-after-an-uncaught-exception">SO</a>
 */
public class UncaughtExceptionHandler implements Thread.UncaughtExceptionHandler {

    @Override
    public void uncaughtException(Thread t, Throwable e) {
        if(e instanceof OutOfMemoryError) {
            System.err.print("Something (probably the code under test or the jvm agent) ran out of memory, which caused an OutOfMemoryError in runner thread (" + t.getName() + ") as well because everything is executed in the same JVM.");
            // because the cause is _probably_ not the runner itself, we return 0 as status code
            System.exit(0);
        } else {
            // something is wrong with the runner
            System.err.print("Uncaught exception in runner thread (" + t.getName() + ").");
            e.printStackTrace();
            System.exit(1);
        }
    }
}


