package de.hsrm.sls.ejunit.ie.master;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import de.hsrm.sls.ejunit.common.TestcaseIdentifier;
import de.hsrm.sls.trex.Testcase;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.lang.management.ManagementFactory;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.concurrent.TimeUnit;

public class WorkerClient {

    private String workerJarPath;
    private int timeout;
    private String jacocoAgentPath;

    public WorkerClient(String workerJarPath, int timeout, String jacocoAgentPath) {
        this.workerJarPath = workerJarPath;
        this.timeout = timeout;
        this.jacocoAgentPath = jacocoAgentPath;
    }

    public int getTimeout() {
        return timeout;
    }

    /**
     * Executes the testcase identified by the given id in a separate JVM by spawning a worker
     * process.
     * <p>
     * This method throws an exception if there was an error while executing the test case. If the
     * testcase does not terminate (due to infinite loops, for example), there is **no error**.
     *
     * @param testcaseId
     * @return Testcase if the testcase has finished within the {@link WorkerClient#timeout},
     * otherwise null
     */
    public Testcase runTest(TestcaseIdentifier testcaseId) {
        String currentClasspath = System.getProperty("java.class.path");

        String javaExecutable = "java";

        List<String> workerArgs = new ArrayList<>();
        workerArgs.add(javaExecutable);
        workerArgs.add("-XX:MaxRAMPercentage=50");
        if (jacocoAgentPath != null) {
            workerArgs.add("-javaagent:%s=destfile=%s.exec".formatted(jacocoAgentPath,
                testcaseId.toString()));
        }

        var mx = ManagementFactory.getRuntimeMXBean();
        var assertionsEnabled = mx.getInputArguments().contains("-ea");
        if (assertionsEnabled) {
            workerArgs.add("-ea");
        }

        var previewFeaturesEnabled = mx.getInputArguments().contains("--enable-preview");
        if (previewFeaturesEnabled) {
            workerArgs.add("--enable-preview");
        }

        workerArgs.add("-cp");
        workerArgs.add(workerJarPath + ":" + currentClasspath);
        workerArgs.add("de.hsrm.sls.ejunit.ie.worker.Worker");

        Process workerProcess = null;
        try {
            var tempFile = File.createTempFile("ejunit-", "");
            tempFile.deleteOnExit();

            workerArgs.add("-clazz");
            workerArgs.add(testcaseId.className());
            workerArgs.add("-method");
            workerArgs.add(testcaseId.methodName());
            workerArgs.add("-out");
            workerArgs.add(tempFile.getPath());

            ProcessBuilder processBuilder = new ProcessBuilder(workerArgs);
            workerProcess = processBuilder.start();
            inheritIO(testcaseId, workerProcess.getInputStream(), System.out);
            inheritIO(testcaseId, workerProcess.getErrorStream(), System.err);
            var hasExited = workerProcess.waitFor(timeout, TimeUnit.SECONDS);
            if(!hasExited) {
                // timeout
                return null;
            }

            if (workerProcess.exitValue() != 0) {
                //throw new RuntimeException(out + err);
                throw new RuntimeException("worker returned exit code != 0");
            }

            var plainResponse = Files.readString(tempFile.toPath());
            var objectMapper = new ObjectMapper();
            objectMapper.registerModule(new JavaTimeModule());
            return objectMapper.readValue(plainResponse, Testcase.class);
        } catch (InterruptedException ex) {
            return null;
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            if(workerProcess != null) {
                workerProcess.destroy();
            }
        }
    }

    /**
     * <a
     * href="https://stackoverflow.com/questions/14165517/processbuilder-forwarding-stdout-and-stderr-of-started-processes-without-blocki">Source</a>
     * Threads will die automatically when subprocess finishes, because src will EOF.
     *
     * @param src
     * @param dest
     */
    private static void inheritIO(TestcaseIdentifier testcaseId, final InputStream src,
        final PrintStream dest) {
        new Thread(() -> {
            var sc = new Scanner(src);
            while (sc.hasNextLine()) {
                dest.println(
                    "[WORKER %s#%s] %s".formatted(testcaseId.className(), testcaseId.methodName(),
                        sc.nextLine()));
            }
        }).start();
    }
}
