package de.hsrm.sls.ejunit.mono;

import java.lang.management.ManagementFactory;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class RunArgs {

    private boolean withJacoco;
    private String outPath;
    private String[] selectors;
    private int maxOutputSize;

    public String[] getSelectors() {
        return selectors;
    }

    public void setSelectors(String[] selectors) {
        this.selectors = selectors;
    }

    public String getOutPath() {
        return outPath;
    }

    public void setOutPath(String outPath) {
        this.outPath = outPath;
    }

    public boolean isWithJacoco() {
        return withJacoco;
    }

    public void setWithJacoco(boolean withJacoco) {
        this.withJacoco = withJacoco;
    }

    public int getMaxOutputSize() {
        return maxOutputSize;
    }

    public void setMaxOutputSize(int maxOutputSize) {
        this.maxOutputSize = maxOutputSize;
    }

    private static final String OPT_JACOCO = "with-jacoco";
    private static final String OPT_OUT_PATH = "out";
    private static final String OPT_MAX_OUTPUT_SIZE = "max-output-size";

    private static final int DEFAULT_MAX_OUTPUT_SIZE = 4096;

    private static CommandLine parseArgs(String[] args) {
        Options options = new Options();

        var jacoco = new Option("jacoco", OPT_JACOCO, false,
            "Enable per-test coverage instrumentation with JaCoCo (online). The jacocoagent.jar needs to be configured using the -javaagent JVM flag. Per-test coverage data will be saved in the working directory after each test in the following format: <testclass>#<testmethod>.exec");
        jacoco.setRequired(false);
        options.addOption(jacoco);

        var outPath = new Option("o", OPT_OUT_PATH, true,
            "Output path for the resulting TREX report.");
        outPath.setRequired(false);
        options.addOption(outPath);

        var maxOutputSize = new Option("s", OPT_MAX_OUTPUT_SIZE, true,
            "Maximum number of characters to include in stdout/stderr outputs from testcases before truncation. If the output is larger, only the last ARG characters are kept and a truncation notice is added to the output. Defaults to %s.".formatted(DEFAULT_MAX_OUTPUT_SIZE));
        maxOutputSize.setRequired(false);
        options.addOption(maxOutputSize);

        CommandLineParser parser = new DefaultParser();
        HelpFormatter formatter = new HelpFormatter();
        CommandLine cmd = null;

        try {
            cmd = parser.parse(options, args);
        } catch (ParseException e) {
            System.out.println(e.getMessage());
            formatter.printHelp(
                "ejunit-mono [OPTIONS] [selector1 [selector2 [selector3] ...]]",
                "Executes all JUnit test methods in the test classes identified by the given selectors in the SAME JVM AS THE TEST RUNNER. A selector is either a FQCN of a test class (all test methods) or an identifier in the format <testclass >#<testmethod> (single test method). Test classes need to be on the class path with the code under test.\n\nOPTIONS:",
                options,
                "\nIf all test methods in the given test classes have finished, exit with status 0. This is independent from the result of the test case, meaning that even if test failures or errors occur, the status is 0. If an unexpected exception occurs in the test runner itself, return status 1.\n\nTest methods that run infinitely (due to infinite loops in the solution, for example) will cause the process to run forever. The caller needs to ensure that the process is killed after an appropriate time span. However, the report contains all test methods and is written to disk on every state change in test execution. This makes it possible to identify the test method that was running indefinitely when the process is killed (which is in state 'started').");
            System.exit(1);
        }

        return cmd;
    }

    public static RunArgs parse(String[] args) {
        var cmd = parseArgs(args);

        var withJacoco = cmd.hasOption(OPT_JACOCO);
        var trexOutPath = cmd.getOptionValue(OPT_OUT_PATH, "trex_report.xml");
        var maxOutputSize = Integer.parseInt(cmd.getOptionValue(OPT_MAX_OUTPUT_SIZE, String.valueOf(DEFAULT_MAX_OUTPUT_SIZE)));

        if (withJacoco && !isJavaAgentConfigured()) {
            throw new IllegalArgumentException(
                "No java agent configured for the current JVM. If %s is set, the jacocoagent.jar needs to be configured as java agent.".formatted(
                    OPT_JACOCO));
        }


        var params = new RunArgs();
        params.setOutPath(trexOutPath);
        params.setWithJacoco(withJacoco);
        params.setSelectors(cmd.getArgs());
        params.setMaxOutputSize(maxOutputSize);
        return params;
    }

    private static boolean isJavaAgentConfigured() {
        var inputArguments = ManagementFactory.getRuntimeMXBean().getInputArguments();

        for (String arg : inputArguments) {
            if (arg.startsWith("-javaagent")) {
                return true;
            }
        }

        return false;
    }
}
