package de.hsrm.sls.trex;

import de.hsrm.sls.ejunit.common.DOMElementWriter;
import java.time.format.DateTimeFormatter;
import java.util.stream.Collectors;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class TREXSerializer {

    private static final String E_TESTSUITE = "testsuite";
    private static final String E_TESTCASE = "testcase";
    private static final String E_FAILURE = "failure";
    private static final String E_PENDING = "pending";
    private static final String E_STARTED = "started";
    private static final String E_ERROR = "error";
    private static final String E_TIMEOUT = "timeout";
    private static final String E_SYSTEM_ERR = "system-err";
    private static final String E_SYSTEM_OUT = "system-out";
    private static final String A_NAME = "name";
    private static final String A_TIME = "time";
    private static final String A_MESSAGE = "message";
    private static final String A_CLASSNAME = "classname";
    private static final String A_TIMESTAMP = "timestamp";
    private static final String A_TIMEOUT = "timeout";

    private final String filePath;

    public TREXSerializer(String filePath) {
        this.filePath = filePath;
    }

    public void serialize(Testsuite state) {
        var doc = getDocumentBuilder().newDocument();
        var rootElement = doc.createElement(E_TESTSUITE);
        doc.appendChild(rootElement);

        final String timestamp = state.getStartedAt().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME);
        rootElement.setAttribute(A_TIMESTAMP, timestamp);

        for (var test : state.getTestcases()) {
            var testcaseElement = serializeTestcase(doc, test);
            rootElement.appendChild(testcaseElement);
        }

        if (state.getFinishedAt() != null) {
            rootElement.setAttribute(A_TIME, String.valueOf(state.getTotalTime()));
        }

        var outElement = doc.createElement(E_SYSTEM_OUT);
        var out = state.getTestcases().stream()
            .filter(t -> t.getStdout() != null)
            .map(t -> t.getStdout())
            .collect(Collectors.joining());
        outElement.appendChild(doc.createCDATASection(out));
        rootElement.appendChild(outElement);

        var errElement = doc.createElement(E_SYSTEM_ERR);
        var err = state.getTestcases().stream()
            .filter(t -> t.getStderr() != null)
            .map(t -> t.getStderr())
            .collect(Collectors.joining());
        errElement.appendChild(doc.createCDATASection(err));
        rootElement.appendChild(errElement);

        new DOMElementWriter().write(doc, filePath);
    }

    private Element serializeTestcase(Document doc, Testcase testcase) {
        var currentTest = doc.createElement(E_TESTCASE);
        currentTest.setAttribute(A_NAME, testcase.getName());
        currentTest.setAttribute(A_CLASSNAME, testcase.getClassName());

        var resultElement = serializeResult(doc, testcase.getResult());
        if(resultElement != null) {
            currentTest.appendChild(resultElement);
        }

        var outElement = doc.createElement(E_SYSTEM_OUT);
        outElement.appendChild(
            doc.createCDATASection(testcase.getStdout() != null ? testcase.getStdout() : ""));
        currentTest.appendChild(outElement);

        var errElement = doc.createElement(E_SYSTEM_ERR);
        errElement.appendChild(
            doc.createCDATASection(testcase.getStderr() != null ? testcase.getStderr() : ""));
        currentTest.appendChild(errElement);

        if (testcase.getFinishedAt() != null) {
            currentTest.setAttribute(A_TIME, String.valueOf(testcase.getTotalTime()));
        }

        return currentTest;
    }

    private Element serializeResult(Document doc, Result result) {
        Element resultElement = null;

        if (result instanceof FailureResult failureResult) {
            resultElement = doc.createElement(E_FAILURE);
            var message = failureResult.getMessage();
            if (message != null && message.length() > 0) {
                resultElement.setAttribute(A_MESSAGE, message);
            }

            Text trace = doc.createTextNode(failureResult.getDetails());
            resultElement.appendChild(trace);
        }

        if (result instanceof ErrorResult errorResult) {
            resultElement = doc.createElement(E_ERROR);
            var message = errorResult.getMessage();
            if (message != null && message.length() > 0) {
                resultElement.setAttribute(A_MESSAGE, message);
            }

            Text trace = doc.createTextNode(errorResult.getDetails());
            resultElement.appendChild(trace);
        }

        if (result instanceof TimeoutResult timeoutResult) {
            resultElement = doc.createElement(E_TIMEOUT);
            resultElement.setAttribute(A_TIMEOUT, String.valueOf(timeoutResult.getTimeout()));
        }

        if (result instanceof PendingResult pendingResult) {
            resultElement = doc.createElement(E_PENDING);
        }

        if (result instanceof StartedResult startedResult) {
            resultElement = doc.createElement(E_STARTED);
        }

        return resultElement;
    }

    private static DocumentBuilder getDocumentBuilder() {
        try {
            return DocumentBuilderFactory.newInstance().newDocumentBuilder();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
