package de.hsrm.sls.trex.junit;

import de.hsrm.sls.ejunit.common.JUnitUtils;
import de.hsrm.sls.ejunit.common.StdCapture;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import org.junit.runner.Description;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunListener;


public class RecordStateRunListener extends RunListener {

    private RunState runState = new RunState();

    private RunStateChangedListener stateChangedListener;

    private int maxOutputSize;

    public RecordStateRunListener(RunStateChangedListener stateChangedListener, int maxOutputSize) {
        this.stateChangedListener = stateChangedListener;
        this.maxOutputSize = maxOutputSize;
    }

    public RecordStateRunListener(int maxOutputSize) {
        this(null, maxOutputSize);
    }

    public RunState getRunState() {
        return runState;
    }

    @Override
    public void testRunStarted(Description descr) {
        runState.setStartedAt(LocalDateTime.now());

        var testcases = new ArrayList<Description>();
        collectTestcases(descr, testcases);

        for (var testcase : testcases) {
            // we ignore error testcases to prevent them from going into the state
            // rethrowing as exception will not propagate because JUnit catches them and creates
            // a failure for that...
            if (JUnitUtils.isJUnitErrorTestcase(testcase)) {
                continue;
            }

            runState.getTests().add(testcase);
            runState.getTestStates().put(testcase, TestcaseState.PENDING);
        }

        if (stateChangedListener != null) {
            stateChangedListener.stateChanged(runState);
        }
    }

    private void collectTestcases(Description descr, List<Description> testcases) {
        if (descr.getMethodName() != null) {
            testcases.add(descr);
        }
        for (var c : descr.getChildren()) {
            collectTestcases(c, testcases);
        }
    }

    @Override
    public void testStarted(Description descr) {
        runState.getTestStates().put(descr, TestcaseState.RUNNING);
        runState.getTestStarts().put(descr, LocalDateTime.now());

        var stdoutCapture = new StdCapture(() -> System.out, System::setOut, maxOutputSize);
        stdoutCapture.startCapture();
        runState.getTestOut().put(descr, stdoutCapture);

        var stderrCapture = new StdCapture(() -> System.err, System::setErr, maxOutputSize);
        stderrCapture.startCapture();
        runState.getTestErr().put(descr, stderrCapture);

        if (stateChangedListener != null) {
            stateChangedListener.stateChanged(runState);
        }
    }


    @Override
    public void testFailure(Failure f) {
        if (JUnitUtils.isJUnitErrorTestcase(f.getDescription())) {
            return;
        }

        if(JUnitUtils.getTestCaseName(f.getDescription()) == null) {
            handleBeforeAllFailure(f);
            return;
        }

        runState.getTestFailures().put(f.getDescription(), f);
    }

    public void testAssumptionFailure(Failure f) {
        if (JUnitUtils.isJUnitErrorTestcase(f.getDescription())) {
            return;
        }

        if(JUnitUtils.getTestCaseName(f.getDescription()) == null) {
            handleBeforeAllFailure(f);
            return;
        }

        runState.getTestFailures().put(f.getDescription(), f);
    }

    /**
     * @BeforeAll executes code before running the testcases. If this code throws an
     * exception, the handlers (testStarted, testFinished) will never be called because the tests are never executed.
     * Such exceptions won't be visible in the results, so we mark all testcases as executed and propagate the failure.
     *
     * Note: @Before works as expected, testStarted and testFinished are called
     */
    private void handleBeforeAllFailure(Failure f) {
        for(var child : f.getDescription().getChildren()) {
            testStarted(child);
            testFailure(new Failure(child, f.getException()));
            testFinished(child);
        }
    }

    @Override
    public void testFinished(Description descr) {
        runState.getTestStates().put(descr, TestcaseState.FINISHED);
        runState.getTestEnds().put(descr, LocalDateTime.now());

        var outCapture = runState.getTestOut().get(descr);
        outCapture.stopCapture();

        var errCapture = runState.getTestErr().get(descr);
        errCapture.stopCapture();

        if (stateChangedListener != null) {
            stateChangedListener.stateChanged(runState);
        }
    }

    @Override
    public void testRunFinished(Result result) {
        System.setOut(System.out);
        runState.setFinishedAt(LocalDateTime.now());

        if (stateChangedListener != null) {
            stateChangedListener.stateChanged(runState);
        }
    }
}
