package de.hsrm.sls.sqlunit;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import de.hsrm.sls.sqlunit.evaluation.QueryEvaluator;
import de.hsrm.sls.sqlunit.evaluation.SQLSyntaxException;
import de.hsrm.sls.sqlunit.segment.Segment;
import de.hsrm.sls.sqlunit.trex.*;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Spliterators;
import java.util.stream.StreamSupport;

public class TestRunner {
    private QueryEvaluator evaluator;
    private ObjectMapper objectMapper;
    private TestRunnerListener listener = testsuite -> {};

    public TestRunner(QueryEvaluator evaluator, ObjectMapper objectMapper) {
        this.evaluator = evaluator;
        this.objectMapper = objectMapper;
    }

    public void setListener(TestRunnerListener listener) {
        this.listener = listener;
    }

    public Testsuite runTests(List<Segment> solutions, List<Segment> specifications) {

        var testsuite = new Testsuite();
        testsuite.setStartedAt(LocalDateTime.now());

        for(var spec : specifications) {
            var testcase = new Testcase("", spec.id());
            testcase.setStartedAt(LocalDateTime.now());
            testcase.setResult(new PendingResult());
            testsuite.getTestcases().add(testcase);
        }

        listener.stateChanged(testsuite);

        for(var spec : specifications) {
            var testcase = testsuite.getTestcases().stream().filter(t -> t.getName().equals(spec.id())).findFirst().get();

            var solution = solutions.stream().filter(e -> e.id().equals(spec.id()))
                .findFirst()
                .orElse(null);
            if(solution == null) {
                var error = new ErrorResult();
                error.setMessage("Keine Lösung angegeben");
                error.setDetails("Es wurde keine Lösung für die Abfrage %s angegeben".formatted(spec.id()));
                testcase.setResult(error);
            } else {
                testcase.setResult(new StartedResult());
                listener.stateChanged(testsuite);

                try {
                    var expected = objectMapper.readTree(spec.content());
                    var rows = evaluator.query(solution.content());
                    var actual = objectMapper.valueToTree(Map.of("result", rows));

                    testcase.setResult(assertResult(expected, actual)
                        ? new SuccessResult()
                        : createFailure(expected, actual));
                } catch (SQLSyntaxException e) {
                    var error = new ErrorResult();
                    error.setMessage("Fehler in der SQL Syntax");
                    error.setDetails(e.getMessage());
                    testcase.setResult(error);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }

            testcase.setFinishedAt(LocalDateTime.now());
            listener.stateChanged(testsuite);
        }

        testsuite.setFinishedAt(LocalDateTime.now());
        listener.stateChanged(testsuite);
        return testsuite;
    }

    private boolean assertResult(JsonNode expected, JsonNode actual) {
        if (isUnsorted(expected)) {
            return sameRowset(expected.get("result"), actual.get("result"));
        } else if (actual.equals(expected)) {
            return true;
        } else {
            return false;
        }
    }

    private boolean isUnsorted(JsonNode expected) {
        JsonNode unsorted = expected.get("unsorted");
        return unsorted != null && unsorted.isBoolean() && unsorted.booleanValue();
    }

    private FailureResult createFailure(JsonNode expected, JsonNode result) {
        var sb = new StringBuilder();
        sb.append("erwartet");
        if(isUnsorted(expected)) {
            sb.append(" (in beliebiger Reihenfolge)");
        }
        sb.append(":\n");
        var e = expected.get("result").toPrettyString();

        sb.append(e.length() > 500 ? (e.substring(0, 500) + "...") : e);

        sb.append("\naber Ergebnis ist:\n");
        var r = result.get("result").toPrettyString();
        sb.append(r.length() > 500 ? (r.substring(0, 500) + "...") : r);

        var failure = new FailureResult();
        failure.setMessage("falsches Ergebnis");
        failure.setDetails(sb.toString());
        return failure;
    }

    private boolean sameRowset(JsonNode left, JsonNode right) {
        if (left instanceof ArrayNode leftArray && right instanceof ArrayNode rightArray) {
            if (rightArray.size() != leftArray.size()) {
                return false;
            }

            var leftElements = StreamSupport.stream(Spliterators.spliteratorUnknownSize(leftArray.elements(), 0), true).toList();
            var rightElements = StreamSupport.stream(Spliterators.spliteratorUnknownSize(rightArray.elements(), 0), true).toList();

            for (JsonNode n : leftElements) {
                if (rightElements.stream().noneMatch(e -> e.equals(n))) {
                    return false;
                }
            }
            for (JsonNode n : rightElements) {
                if (leftElements.stream().noneMatch(e -> e.equals(n))) {
                    return false;
                }
            }
            return true;
        }

        return false;
    }
}
