package de.hsrm.sls.sqlunit.evaluation;

import org.antlr.v4.runtime.CharStreams;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.tree.ParseTreeWalker;

import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.SQLSyntaxErrorException;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Stream;

/**
 * currently not in use - maybe necessary some day if removal of comments cannot be done properly in {@link SimpleQueryEvaluator}
 */
public class ANTLRQueryEvaluator implements QueryEvaluator {

  private SQLConfig config;

  public ANTLRQueryEvaluator(SQLConfig config) {
    this.config = config;
  }

  /**
   * Executes the query on the database and returns the result as JSON. When using multiple statements
   * in the same query, only the last result will be returned.
   *
   * @param query SQL query
   * @return list of resulting rows as JSON
   */
  @Override
  public List<Map<String, Object>> query(String query) {
    try {
      Class.forName("org.mariadb.jdbc.Driver");
      var connection = DriverManager.getConnection(config.url());

      var rows = new ArrayList<Map<String, Object>>();
      var rawStatements = splitStatements(query);
      for (var s : rawStatements) {
        var statement = connection.createStatement();

        var rs = statement.executeQuery(s);
        var md = rs.getMetaData();
        var cns = new TreeMap<Integer, String>();
        for (int i = 1; i <= md.getColumnCount(); i++) {
          cns.put(i, md.getColumnLabel(i));
        }

        rows = new ArrayList<>();
        while (rs.next()) {
          var map = new HashMap<String, Object>();
          for (var entry : cns.entrySet()) {
            map.put(entry.getValue(), rs.getObject(entry.getKey()) + "");
          }
          rows.add(map);
        }
      }

      return rows;
    } catch(SQLSyntaxErrorException e) {
      throw new SQLSyntaxException(e.getMessage(), e);
    } catch (SQLException | ClassNotFoundException e) {
        throw new RuntimeException(e);
    }
  }

  /**
   * Extracts the statements from the query, e.g.:
   *
   * q:
   * SELECT * FROM person;
   * SELECT * FROM person LIMIT 1;
   *
   * will yield:
   * [0] SELECT * FROM person;
   * [1] SELECT * FROM person LIMIT 1;
   *
   * Supports the DELIMITER statements that is used in the MariaDB CLI client for defining complex
   * queries, such as triggers. JDBC does not support the DELIMITER statement, so we need to handle it.
   *
   * For example:
   *
   * q:
   * DELIMITER //
   *
   * CREATE FUNCTION FortyTwo() RETURNS TINYINT DETERMINISTIC
   * BEGIN
   *   DECLARE x TINYINT;
   *   SET x = 42;
   *   RETURN x;
   * END
   *
   * //
   *
   * DELIMITER ;
   *
   * will yield:
   *
   * [0]: CREATE FUNCTION FortyTwo() RETURNS TINYINT DETERMINISTIC
   * BEGIN
   *   DECLARE x TINYINT;
   *   SET x = 42;
   *   RETURN x;
   * END
   *
   * @param q query containing multiple statements
   * @return statements
   */
  private List<String> splitStatements(String q) {
    var pattern = Pattern.compile("delimiter (.*)");
    var matcher = pattern.matcher(q);
    var matches = matcher.results().toList();

    // no DELIMITER statement, we just use the default ; for splitting
    if (matches.isEmpty()) {
      return splitStatements(q, ";");
    }

    // otherwise apply the delimiter specified with DELIMITER for each "section"
    // since the delimiter can be changed multiple times in the same query
    var queryParts = new ArrayList<String>();
    for (var i = 0; i < matches.size(); i++) {
      var match = matches.get(i);
      var nextMatch = i + 1 < matches.size() ? matches.get(i + 1) : null;

      var start = match.end();
      var end = nextMatch != null ? nextMatch.start() : q.length();
      var s = q.substring(start, end);

      var delim = match.group(1);
      queryParts.addAll(splitStatements(s, delim));
    }

    return queryParts;
  }

  private List<String> splitStatements(String q, String delim) {
    return Stream.of(q.split(delim)).map(String::trim)
            .filter(s -> !s.isEmpty())
            .map(this::parseStatement)
            .filter(Objects::nonNull)
            .toList();
  }

  /**
   * parses the statement using the ANTLR MariaDB parser and returns the original statement WITHOUT COMMENTS.
   * Comments need to be removed as they evaluate to an empty result set and would overwrite the result of a previous query.
   * @param s containing ONE SINGLE statement
   * @return original statement without comments or null if the statement only consists of a comment
   */
  private String parseStatement(String s) {
    var stream = CharStreams.fromString(s);
    var lexer = new MariaDBLexer(stream);
    var tokens = new CommonTokenStream(lexer);

    var parser = new MariaDBParser(tokens);
    var errorListener = new SyntaxErrorListener();
    parser.addErrorListener(errorListener);

    var tree = parser.root();

    if(!errorListener.getSyntaxErrors().isEmpty()) {
      throw new SQLSyntaxException(errorListener.getSyntaxErrors().get(0).getMessage());
    }

    var walker = new ParseTreeWalker();
    var listener = new ExtractStatementsListener(stream);
    walker.walk(listener, tree);

    var statements = listener.getStatements();
    return !statements.isEmpty() ? statements.get(0) : null;
  }
}
