package de.hsrm.sls.sqlunit;

import static org.junit.Assert.assertEquals;
import static org.springframework.util.Assert.isInstanceOf;

import ch.vorburger.mariadb4j.DBConfigurationBuilder;
import ch.vorburger.mariadb4j.junit.MariaDB4jRule;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.sqlunit.evaluation.SQLConfig;
import de.hsrm.sls.sqlunit.evaluation.SimpleQueryEvaluator;
import de.hsrm.sls.sqlunit.segment.Segment;
import de.hsrm.sls.sqlunit.trex.ErrorResult;
import de.hsrm.sls.sqlunit.trex.FailureResult;
import de.hsrm.sls.sqlunit.trex.SuccessResult;

import java.util.List;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Test;

public class TestRunnerTest {
  @ClassRule
  public static CustomMariaDB4jRule dbRule = new CustomMariaDB4jRule(
      DBConfigurationBuilder.newBuilder().setPort(0).build(),
      "test",
      "import.sql"
  );
  TestRunner testRunner;

  @Before
  public void setUp() {
    var evaluator = new SimpleQueryEvaluator(new SQLConfig(dbRule.getURL()));
    testRunner = new TestRunner(evaluator, new ObjectMapper());
  }

  @Test
  public void test1() {
    var solutions = List.of(
        new Segment("q1", "SELECT * FROM person WHERE name LIKE 'K%' ORDER BY p_nr ASC;")
    );
    var tests = List.of(
        new Segment("q1", "{\"result\": [ { \"p_nr\" : \"P023\", \"vorname\" : \"Jan\", \"name\" : \"Kramer\" },  { \"p_nr\" : \"P017\", \"vorname\" : \"Keira\", \"name\" : \"Knightley\" }, {\"p_nr\": \"P002\" ,\"vorname\": \"Marion\" ,\"name\": \"Küstenmacher\" }, {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(FailureResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test2() {
    var solutions = List.of(
        new Segment("q1", "SELECT * FROM person WHERE name LIKE 'K%' ORDER BY p_nr ASC;")
    );
    var tests = List.of(
        new Segment("q1", "{\"unsorted\": false, \"result\": [ { \"p_nr\" : \"P023\", \"vorname\" : \"Jan\", \"name\" : \"Kramer\" },  { \"p_nr\" : \"P017\", \"vorname\" : \"Keira\", \"name\" : \"Knightley\" }, {\"p_nr\": \"P002\" ,\"vorname\": \"Marion\" ,\"name\": \"Küstenmacher\" }, {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(FailureResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test3() {
    var solutions = List.of(
        new Segment("q1", "SELECT * FROM person WHERE name LIKE 'K%' ORDER BY p_nr DESC;")
    );
    var tests = List.of(
        new Segment("q1", "{\"result\": [ { \"p_nr\" : \"P023\", \"vorname\" : \"Jan\", \"name\" : \"Kramer\" },  { \"p_nr\" : \"P017\", \"vorname\" : \"Keira\", \"name\" : \"Knightley\" }, {\"p_nr\": \"P002\" ,\"vorname\": \"Marion\" ,\"name\": \"Küstenmacher\" }, {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(SuccessResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test4() {
    var solutions = List.of(
        new Segment("q1", "SELECT * FROM person WHERE name LIKE 'K%' ORDER BY p_nr ASC;")
    );
    var tests = List.of(
        new Segment("q1", "{\"unsorted\": true, \"result\": [ { \"p_nr\" : \"P023\", \"vorname\" : \"Jan\", \"name\" : \"Kramer\" },  { \"p_nr\" : \"P017\", \"vorname\" : \"Keira\", \"name\" : \"Knightley\" }, {\"p_nr\": \"P002\" ,\"vorname\": \"Marion\" ,\"name\": \"Küstenmacher\" }, {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(SuccessResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test5() {
    var solutions = List.of(
        new Segment("q1", "SELECT * FROM person WHERE name = 'TTT';")
    );
    var tests = List.of(
        new Segment("q1", "{\"result\": [ {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(FailureResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test6() {
    var solutions = List.of(
        new Segment("q1", "SELECTasd * FROM person WHERE name = 'TTT';")
    );
    var tests = List.of(
        new Segment("q1", "{\"result\": [ {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(ErrorResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test7() {
    var solutions = List.of(
        new Segment("q1", "SELECT * FROM person2 WHERE name = 'TTT';")
    );
    var tests = List.of(
        new Segment("q1", "{\"result\": [ {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(ErrorResult.class, testsuite.getTestcases().get(0).getResult());
  }

  @Test
  public void test8() {
    List<Segment> solutions = List.of();
    var tests = List.of(
        new Segment("q1", "{\"result\": [ {\"p_nr\": \"P001\" ,\"vorname\": \"Hape\" ,\"name\": \"Kerkeling\" } ] }")
    );

    var testsuite = testRunner.runTests(solutions, tests);

    assertEquals(1,testsuite.getTestcases().size());
    isInstanceOf(ErrorResult.class, testsuite.getTestcases().get(0).getResult());
  }
}
