package de.hsrm.sls.sqlunit.evaluation;

import static org.junit.Assert.assertEquals;

import ch.vorburger.mariadb4j.DBConfigurationBuilder;
import de.hsrm.sls.sqlunit.CustomMariaDB4jRule;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Test;

public class QueryEvaluatorTest {
  @ClassRule
  public static CustomMariaDB4jRule dbRule = new CustomMariaDB4jRule(
      DBConfigurationBuilder.newBuilder().setPort(0).build(),
      "test",
      "import.sql"
  );
  private QueryEvaluator evaluator;

  @Before
  public void setUp() {
    evaluator = new SimpleQueryEvaluator(new SQLConfig(dbRule.getURL()));
  }

  @Test
  public void testDelimiter() {
    var query = """
delimiter //

CREATE OR REPLACE TRIGGER tr_in_lager
AFTER INSERT ON
bestellung_beinhaltet_medienartikel
FOR EACH ROW
BEGIN
	UPDATE lagerbestand SET bestand = bestand - NEW.anzahl
	WHERE a_nr=NEW.a_nr;
END;
//

CREATE OR REPLACE TRIGGER tr_up_lager
AFTER UPDATE ON
bestellung_beinhaltet_medienartikel
FOR EACH ROW
BEGIN
	UPDATE lagerbestand SET bestand = bestand + OLD.anzahl - NEW.anzahl
	WHERE a_nr=NEW.a_nr;
END;
//

delimiter ;
""";

    var rows = evaluator.query(query);
    assertEquals(0,rows.size());
  }

  @Test
  public void testSelect() {
    var query = """
SELECT * FROM person
WHERE name = 'Kerkeling';
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("name"));
  }

  @Test
  public void testMultiSelect() {
    var query = """
SELECT * FROM person WHERE name = 'Kerkeling';
SELECT * FROM person WHERE name = 'Küstenmacher';
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Marion", row.get("vorname"));
    assertEquals("Küstenmacher", row.get("name"));
  }

  @Test
  public void testSelectDoubleSemicolon() {
    var query = """
SELECT * FROM person
WHERE name = 'Kerkeling';;
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("name"));
  }

  @Test
  public void testSelectWithComment1() {
    var query = """
SELECT * FROM person WHERE name = 'Kerkeling'; -- alternativ: LENGTH(id) >= 5
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("name"));
  }

  @Test
  public void testSelectWithComment2() {
    var query = """
SELECT * FROM person WHERE name = 'Kerkeling'; -- alternativ: LENGTH(id) >= 5;
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("name"));
  }

  @Test
  public void testSelectWithComment3() {
    var query = """
SELECT * FROM person WHERE name = 'Kerkeling';
-- alternativ: LENGTH(id) >= 5;
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("name"));
  }

  @Test
  public void testSelectWithComment4() {
    var query = """
SELECT * FROM person WHERE name = 'Kerkeling'; # alternativ: LENGTH(name) >= 5;
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("name"));
  }

  @Test
  public void testInsert() {
    var query = """
INSERT INTO person VALUES ('P111', 'Eva', 'Luator');
""";

    var rows = evaluator.query(query);
    assertEquals(0,rows.size());
  }

  @Test
  public void testMultiInsert() {
    var query = """
INSERT INTO person VALUES ('P112', 'Evalina', 'Luatino');
INSERT INTO person VALUES ('P113', 'Evalos', 'Luatos')
""";

    var rows = evaluator.query(query);
    assertEquals(0,rows.size());
  }

  @Test(expected = SQLSyntaxException.class)
  public void testInvalidSyntax() {
    var query = """
SELECTasads * FROM person
WHERE name = 'Kerkeling';
""";

    evaluator.query(query);
  }

  @Test(expected = SQLSyntaxException.class)
  public void testInvalidSyntax2() {
    var query = """
SELECT * FROM person2
WHERE name = 'Kerkeling';
""";

    evaluator.query(query);
  }

  @Test
  public void testAlias() {
    var query = """
SELECT vorname, name as nameAlias FROM person
WHERE name = 'Kerkeling';
""";

    var rows = evaluator.query(query);
    assertEquals(1,rows.size());

    var row = rows.get(0);
    assertEquals("Hape", row.get("vorname"));
    assertEquals("Kerkeling", row.get("nameAlias"));
  }
}
